@@ -54,7 +54,14 @@ REveScene::~REveScene()
    REX::gEve->GetViewers()->SceneDestructing(this);
    REX::gEve->GetScenes()->RemoveElement(this);
 }
+//------------------------------------------------------------------------------
 
+int REveScene::WriteCoreJson(nlohmann::json &j, Int_t rnr_offset)
+{
+   j["Mandatory"] = fMandatory;
+
+   return REveElement::WriteCoreJson(j, rnr_offset);
+}
 //------------------------------------------------------------------------------
 
 void REveScene::AddSubscriber(std::unique_ptr<REveClient> &&sub)
@@ -70,8 +77,6 @@ void REveScene::AddSubscriber(std::unique_ptr<REveClient> &&sub)
 
 void REveScene::RemoveSubscriber(unsigned id)
 {
-   assert(fAcceptingChanges == kFALSE);
-
    auto pred = [&](std::unique_ptr<REveClient> &client) {
       return client->fId == id;
    };
@@ -99,9 +104,11 @@ void REveScene::BeginAcceptingChanges()
 
    if (HasSubscribers()) {
       fAcceptingChanges = kTRUE;
+      /*
       for (auto &&client : fSubscribers) {
          REX::gEve->SceneSubscriberProcessingChanges(client->fId);
       }
+      */
    }
 }
 
@@ -124,15 +131,6 @@ void REveScene::EndAcceptingChanges()
    fAcceptingChanges = kFALSE;
 }
 
-void REveScene::ProcessChanges()
-{
-   if (IsChanged())
-   {
-      StreamRepresentationChanges();
-      SendChangesToSubscribers();
-   }
-}
-
 void REveScene::StreamElements()
 {
    fOutputJson.clear();
@@ -147,7 +145,7 @@ void REveScene::StreamElements()
    jhdr["content"]  = "REveScene::StreamElements";
    jhdr["fSceneId"] = fElementId;
 
-   if (fCommands.size() > 0) {
+   if (!fCommands.empty()) {
       jhdr["commands"] = nlohmann::json::array();
       for (auto &&cmd : fCommands) {
          nlohmann::json jcmd = {};
@@ -235,9 +233,6 @@ void REveScene::StreamJsonRecurse(REveElement *el, nlohmann::json &jarr)
 
 void REveScene::StreamRepresentationChanges()
 {
-   fOutputJson.clear();
-   fOutputBinary.clear();
-
    fElsWithBinaryData.clear();
    fTotalBinarySize = 0;
 
@@ -327,9 +322,11 @@ void REveScene::StreamRepresentationChanges()
 void REveScene::SendChangesToSubscribers()
 {
    for (auto && client : fSubscribers) {
-      if (gDebug > 0)
-         printf("   sending json, len = %d --> to conn_id = %d\n", (int) fOutputJson.size(), client->fId);
-      client->fWebWindow->Send(client->fId, fOutputJson);
+      if (!fOutputJson.empty()) {
+         if (gDebug > 0)
+            printf("   sending json, len = %d --> to conn_id = %d\n", (int) fOutputJson.size(), client->fId);
+         client->fWebWindow->Send(client->fId, fOutputJson);
+      }
       if (fTotalBinarySize) {
          if (gDebug > 0)
             printf("   sending binary, len = %d --> to conn_id = %d\n", fTotalBinarySize, client->fId);
@@ -337,6 +334,9 @@ void REveScene::SendChangesToSubscribers()
       }
       REX::gEve->SceneSubscriberWaitingResponse(client->fId);
    }
+   fOutputJson.clear();
+   fOutputBinary.clear();
+   fTotalBinarySize = 0;
 }
 
 Bool_t REveScene::IsChanged() const
@@ -564,21 +564,3 @@ bool REveSceneList::AnyChanges() const
    }
    return false;
 }
-
-
-////////////////////////////////////////////////////////////////////////////////
-//
-// Send an update of element representations
-//
-////////////////////////////////////////////////////////////////////////////////
-
-void REveSceneList::ProcessSceneChanges()
-{
-   if (gDebug > 0)
-      ::Info("REveSceneList::ProcessSceneChanges","processing");
-
-   for (auto &el : fChildren)
-   {
-      ((REveScene*) el)->ProcessChanges();
-   }
-}
