@@ -8,7 +8,7 @@
  * For the list of contributors see $ROOTSYS/README/CREDITS.          *
  *                                                                    *
  **********************************************************************/
-//////////////////////////////////////////////////////////////////////////////
+
 /** \class TKDE
     \ingroup Hist
  Kernel Density Estimation class.
@@ -27,6 +27,7 @@
 
 #include <functional>
 #include <algorithm>
+#include <memory>
 #include <numeric>
 #include <limits>
 #include <cassert>
@@ -43,9 +44,6 @@
 #include "TVirtualPad.h"
 #include "TKDE.h"
 
-// for make_unique
-#include "ROOT/RMakeUnique.hxx"
-
 ClassImp(TKDE);
 
 
@@ -92,12 +90,12 @@ TKDE::~TKDE() {
 //// Internal function to instantiate a TKDE object
 void TKDE::Instantiate(KernelFunction_Ptr kernfunc, UInt_t events, const Double_t* data, const Double_t* dataWeights, Double_t xMin, Double_t xMax, const Option_t* option, Double_t rho) {
 
-   fPDF = 0;
-   fKernelFunction = 0;
-   fUpperPDF = 0;
-   fLowerPDF = 0;
-   fApproximateBias = 0;
-   fGraph = 0;
+   fPDF = nullptr;
+   fKernelFunction = nullptr;
+   fUpperPDF = nullptr;
+   fLowerPDF = nullptr;
+   fApproximateBias = nullptr;
+   fGraph = nullptr;
    fNewData = false;
    fUseMirroring = false; fMirrorLeft = false; fMirrorRight = false;
    fAsymLeft = false; fAsymRight = false;
@@ -486,7 +484,7 @@ void TKDE::ReInit() {
       return;
    }
 
-   if (fEvents.size() == 0) {
+   if (fEvents.empty()) {
       Error("ReInit","TKDE does not contain any data !");
       return;
    }
@@ -891,7 +889,7 @@ void TKDE::SetBinCountData() {
 //                  - Extra options can be passed in opt for the drawing of the corresponding TF1 or TGraph
 //
 // NOTE:  The functions GetDrawnFunction(), GetDrawnUpperFunction(), GetDrawnLowerFunction()
-//  and GetGraphWithErrors() return the corresponding drawn objects (which are maneged by the TKDE)
+//  and GetGraphWithErrors() return the corresponding drawn objects (which are managed by the TKDE)
 // They can be used to changes style, color, etc..
 ////
 void TKDE::Draw(const Option_t* opt) {
@@ -914,7 +912,7 @@ void TKDE::Draw(const Option_t* opt) {
       Double_t level = 0.95;
       const char * s = strstr(plotOpt.Data(),"interval@");
       // coverity [secure_coding : FALSE]
-      if (s != 0) sscanf(s,"interval@%lf",&level);
+      if (s != nullptr) sscanf(s,"interval@%lf",&level);
       if((level <= 0) || (level >= 1)) {
          Warning("Draw","given confidence level %.3lf is invalid - use default 0.95",level);
          level = 0.95;
@@ -929,7 +927,7 @@ void TKDE::Draw(const Option_t* opt) {
 }
 
 ///////////////////////////////////////////////////////////////////////
-///  Draws a TGraphErrors wih KDE values and errors
+///  Draws a TGraphErrors with KDE values and errors
 void TKDE::DrawErrors(TString& drawOpt)
 {
    if (fGraph) delete fGraph;
@@ -993,7 +991,7 @@ const Double_t *  TKDE::GetAdaptiveWeights() const {
    // Returns the bandwidths for the adaptive KDE
    if (fIteration != TKDE::kAdaptive) {
       this->Warning("GetFixedWeight()", "Adaptive iteration option not enabled. Returning a NULL pointer<");
-      return 0;
+      return nullptr;
    }
    if (fNewData) (const_cast<TKDE*>(this))->InitFromNewData();
    return &(fKernel->GetAdaptiveWeights()).front();
@@ -1160,7 +1158,7 @@ Double_t TKDE::ComputeKernelIntegral() const {
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////
-/// Internal funciton to compute statistics (mean,stddev) using always all the provided data (i.e. no binning)
+/// Internal function to compute statistics (mean,stddev) using always all the provided data (i.e. no binning)
 void TKDE::ComputeDataStats() {
    /// in case of weights use
    if (!fEventWeights.empty() ) {
