@@ -68,8 +68,6 @@
 
 ClassImp(TSVDUnfold);
 
-using namespace std;
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Alternative constructor
 /// User provides data and MC test spectra, as well as detector response matrix, diagonal covariance matrix of measured spectrum built from the uncertainties on measured spectrum
@@ -80,16 +78,16 @@ fNdim       (0),
 fDdim       (2),
 fNormalize  (kFALSE),
 fKReg       (-1),
-fDHist      (NULL),
-fSVHist     (NULL),
-fXtau       (NULL),
-fXinv       (NULL),
+fDHist      (nullptr),
+fSVHist     (nullptr),
+fXtau       (nullptr),
+fXinv       (nullptr),
 fBdat       (bdat),
 fBini       (bini),
 fXini       (xini),
 fAdet       (Adet),
-fToyhisto   (NULL),
-fToymat     (NULL),
+fToyhisto   (nullptr),
+fToymat     (nullptr),
 fToyMode    (kFALSE),
 fMatToyMode (kFALSE)
 {
@@ -133,17 +131,17 @@ fNdim       (0),
 fDdim       (2),
 fNormalize  (kFALSE),
 fKReg       (-1),
-fDHist      (NULL),
-fSVHist     (NULL),
-fXtau       (NULL),
-fXinv       (NULL),
+fDHist      (nullptr),
+fSVHist     (nullptr),
+fXtau       (nullptr),
+fXinv       (nullptr),
 fBdat       (bdat),
 fBcov       (Bcov),
 fBini       (bini),
 fXini       (xini),
 fAdet       (Adet),
-fToyhisto   (NULL),
-fToymat     (NULL),
+fToyhisto   (nullptr),
+fToymat     (nullptr),
 fToyMode    (kFALSE),
 fMatToyMode (kFALSE)
 {
@@ -201,37 +199,37 @@ TSVDUnfold::~TSVDUnfold()
 {
    if(fToyhisto){
       delete fToyhisto;
-      fToyhisto = 0;
+      fToyhisto = nullptr;
    }
 
    if(fToymat){
       delete fToymat;
-      fToymat = 0;
+      fToymat = nullptr;
    }
 
    if(fDHist){
       delete fDHist;
-      fDHist = 0;
+      fDHist = nullptr;
    }
 
    if(fSVHist){
       delete fSVHist;
-      fSVHist = 0;
+      fSVHist = nullptr;
    }
 
    if(fXtau){
       delete fXtau;
-      fXtau = 0;
+      fXtau = nullptr;
    }
 
    if(fXinv){
       delete fXinv;
-      fXinv = 0;
+      fXinv = nullptr;
    }
 
    if(fBcov){
       delete fBcov;
-      fBcov = 0;
+      fBcov = nullptr;
    }
 }
 
@@ -252,7 +250,7 @@ TH1D* TSVDUnfold::Unfold( Int_t kreg )
    Double_t eps = 1e-12;
    Double_t sreg;
 
-   // Copy histogams entries into vector
+   // Copy histograms entries into vector
    if (fToyMode) { H2V( fToyhisto, vb ); H2Verr( fToyhisto, vberr ); }
    else          { H2V( fBdat,     vb ); H2Verr( fBdat,     vberr ); }
 
@@ -409,7 +407,7 @@ TH1D* TSVDUnfold::Unfold( Int_t kreg )
 TH2D* TSVDUnfold::GetUnfoldCovMatrix( const TH2D* cov, Int_t ntoys, Int_t seed )
 {
    fToyMode = true;
-   TH1D* unfres = 0;
+   TH1D* unfres = nullptr;
    TH2D* unfcov = (TH2D*)fAdet->Clone("unfcovmat");
    unfcov->SetTitle("Toy covariance matrix");
    for(int i=1; i<=fNdim; i++)
@@ -468,7 +466,7 @@ TH2D* TSVDUnfold::GetUnfoldCovMatrix( const TH2D* cov, Int_t ntoys, Int_t seed )
          toymean->SetBinContent(j, toymean->GetBinContent(j) + unfres->GetBinContent(j)/ntoys);
       }
       delete unfres;
-      unfres = 0;
+      unfres = nullptr;
    }
 
    // Reset the random seed
@@ -497,7 +495,7 @@ TH2D* TSVDUnfold::GetUnfoldCovMatrix( const TH2D* cov, Int_t ntoys, Int_t seed )
          }
       }
       delete unfres;
-      unfres = 0;
+      unfres = nullptr;
    }
    delete Lt;
    delete toymean;
@@ -515,7 +513,7 @@ TH2D* TSVDUnfold::GetUnfoldCovMatrix( const TH2D* cov, Int_t ntoys, Int_t seed )
 TH2D* TSVDUnfold::GetAdetCovMatrix( Int_t ntoys, Int_t seed )
 {
    fMatToyMode = true;
-   TH1D* unfres = 0;
+   TH1D* unfres = nullptr;
    TH2D* unfcov = (TH2D*)fAdet->Clone("unfcovmat");
    unfcov->SetTitle("Toy covariance matrix");
    for(int i=1; i<=fNdim; i++)
@@ -544,7 +542,7 @@ TH2D* TSVDUnfold::GetAdetCovMatrix( Int_t ntoys, Int_t seed )
          toymean->SetBinContent(j, toymean->GetBinContent(j) + unfres->GetBinContent(j)/ntoys);
       }
       delete unfres;
-      unfres = 0;
+      unfres = nullptr;
    }
 
    // Reset the random seed
@@ -566,7 +564,7 @@ TH2D* TSVDUnfold::GetAdetCovMatrix( Int_t ntoys, Int_t seed )
          }
       }
       delete unfres;
-      unfres = 0;
+      unfres = nullptr;
    }
    delete toymean;
    fMatToyMode = kFALSE;
