@@ -389,7 +389,11 @@ namespace cling {
       }
     }
 
-    auto* NewCS = CompoundStmt::Create(*m_Context, NewChildren,
+    FPOptionsOverride FPFeatures;
+    if (Node->hasStoredFPFeatures()) {
+      FPFeatures = Node->getStoredFPFeatures();
+    }
+    auto* NewCS = CompoundStmt::Create(*m_Context, NewChildren, FPFeatures,
                                        Node->getLBracLoc(),
                                        Node->getRBracLoc());
 
@@ -814,7 +818,7 @@ namespace cling {
                                                              ArrayType::Normal,
                                                           /*IndexTypeQuals=*/0);
 
-    StringLiteral::StringKind Kind = StringLiteral::Ascii;
+    StringLiteral::StringKind Kind = StringLiteral::Ordinary;
     Expr* Result = StringLiteral::Create(*m_Context,
                                          Value,
                                          Kind,
@@ -845,12 +849,14 @@ namespace cling {
     // Before instantiation we need the canonical type
     TemplateArgument Arg(InstTy.getCanonicalType());
     TemplateArgumentList TemplateArgs(TemplateArgumentList::OnStack, Arg);
+    MultiLevelTemplateArgumentList MLTAL(m_EvalDecl, TemplateArgs.asArray(),
+                                         /*Final=*/false);
 
     // Substitute the declaration of the templated function, with the
     // specified template argument
     Decl* D = m_Sema->SubstDecl(m_EvalDecl,
                                 m_EvalDecl->getDeclContext(),
-                                MultiLevelTemplateArgumentList(TemplateArgs));
+                                MLTAL);
 
     FunctionDecl* Fn = dyn_cast<FunctionDecl>(D);
 
