@@ -22,9 +22,15 @@
 
 #include "gtest/gtest.h"
 
+// Backward compatibility for gtest version < 1.10.0
+#ifndef TYPED_TEST_SUITE
+#define TYPED_TEST_SUITE TYPED_TEST_CASE
+#endif
+
 #include <chrono>
 #include <iostream>
 #include <string>
+#include <iomanip>
 
 int printLevel = 0;
 
@@ -325,7 +331,7 @@ struct Chi2GradientTestEvaluation : public GradientTestEvaluation<T, ROOT::Fit::
 
    Chi2GradientTestEvaluation() { SetFitter(); }
 
-   virtual void SetFitter() override
+   void SetFitter() override
    {
       this->fFitter = new ROOT::Fit::Chi2FCN<GradFunctionType, BaseFunctionType>(*(this->fData), *(this->fFitFunction),
                                                                                  T::ExecutionPolicyType());
@@ -340,7 +346,7 @@ struct PoissonLikelihoodGradientTestEvaluation : public GradientTestEvaluation<T
 
    PoissonLikelihoodGradientTestEvaluation() { SetFitter(); }
 
-   virtual void SetFitter() override
+   void SetFitter() override
    {
       this->fFitter = new ROOT::Fit::PoissonLikelihoodFCN<GradFunctionType, BaseFunctionType>(
          *(this->fData), *(this->fFitFunction), 0, true, T::ExecutionPolicyType());
@@ -355,7 +361,7 @@ struct LogLikelihoodGradientTestEvaluation : public GradientTestEvaluation<T, RO
 
    LogLikelihoodGradientTestEvaluation() { SetFitter(); }
 
-   virtual void SetFitter() override
+   void SetFitter() override
    {
       this->fFitter = new ROOT::Fit::LogLikelihoodFCN<GradFunctionType, BaseFunctionType>(
          *(this->fData), *(this->fFitFunction), 0, false, T::ExecutionPolicyType());
@@ -372,7 +378,7 @@ class Chi2GradientTest : public ::testing::Test, public Chi2GradientTestEvaluati
    using typename Chi2GradientTestEvaluation<T>::ScalarSerial;
 
 protected:
-   virtual void SetUp()
+   void SetUp() override
    {
       T::PrintTypeInfo("Chi2FCN");
 
@@ -397,7 +403,7 @@ class PoissonLikelihoodGradientTest : public ::testing::Test, public PoissonLike
    using typename PoissonLikelihoodGradientTestEvaluation<T>::ScalarSerial;
 
 protected:
-   virtual void SetUp()
+   void SetUp() override
    {
       T::PrintTypeInfo("PoissonLikelihoodFCN");
 
@@ -422,7 +428,7 @@ class LogLikelihoodGradientTest : public ::testing::Test, public LogLikelihoodGr
    using typename LogLikelihoodGradientTestEvaluation<T>::ScalarSerial;
 
 protected:
-   virtual void SetUp()
+   void SetUp() override
    {
       T::PrintTypeInfo("LogLikelihoodFCN");
 
