@@ -50,11 +50,11 @@ void RosenBrock(Int_t &, Double_t *, Double_t &f, Double_t *par, Int_t /*iflag*/
 class RosenBrockFunction : public ROOT::Math::IMultiGenFunction {
 
 public:
-   virtual ~RosenBrockFunction() {}
+   ~RosenBrockFunction() override {}
 
-   unsigned int NDim() const { return 2; }
+   unsigned int NDim() const override { return 2; }
 
-   ROOT::Math::IMultiGenFunction *Clone() const { return new RosenBrockFunction(); }
+   ROOT::Math::IMultiGenFunction *Clone() const override { return new RosenBrockFunction(); }
 
    const double *TrueMinimum() const
    {
@@ -64,7 +64,7 @@ public:
    }
 
 private:
-   inline double DoEval(const double *x) const
+   inline double DoEval(const double *x) const override
    {
 #ifdef USE_FREE_FUNC
       double f = 0;
@@ -120,9 +120,9 @@ public:
       v0 = A * sx0 + B * cx0;
    }
 
-   unsigned int NDim() const { return fDim; }
+   unsigned int NDim() const override { return fDim; }
 
-   ROOT::Math::IMultiGenFunction *Clone() const
+   ROOT::Math::IMultiGenFunction *Clone() const override
    {
       TrigoFletcherFunction *f = new TrigoFletcherFunction(*this);
       //       std::cerr <<"cannot clone this function" << std::endl;
@@ -149,7 +149,7 @@ public:
 
    const double *TrueMinimum() const { return x0.GetMatrixArray(); }
 
-   void Gradient(const double *x, double *g) const
+   void Gradient(const double *x, double *g) const override
    {
       gNCall2++;
 
@@ -199,7 +199,7 @@ private:
    //    TrigoFletcherFunction(const TrigoFletcherFunction & ) {}
    //    TrigoFletcherFunction & operator=(const TrigoFletcherFunction &) { return *this; }
 
-   double DoEval(const double *x) const
+   double DoEval(const double *x) const override
    {
       gNCall++;
 
@@ -214,7 +214,7 @@ private:
       return r * r;
    }
 
-   double DoDerivative(const double *x, unsigned int i) const
+   double DoDerivative(const double *x, unsigned int i) const override
    {
       std::vector<double> g(fDim);
       Gradient(x, &g[0]);
@@ -243,9 +243,9 @@ class ChebyQuadFunction : public ROOT::Math::IMultiGradFunction {
 public:
    ChebyQuadFunction(unsigned int n) : fDim(n), fvec(std::vector<double>(n)), fTrueMin(std::vector<double>(n)) {}
 
-   unsigned int NDim() const { return fDim; }
+   unsigned int NDim() const override { return fDim; }
 
-   ROOT::Math::IMultiGenFunction *Clone() const { return new ChebyQuadFunction(*this); }
+   ROOT::Math::IMultiGenFunction *Clone() const override { return new ChebyQuadFunction(*this); }
 
    const double *TrueMinimum() const { return &fTrueMin[0]; }
 
@@ -260,7 +260,7 @@ public:
 
    // compute gradient
 
-   void Gradient(const double *x, double *g) const
+   void Gradient(const double *x, double *g) const override
    {
       gNCall2++;
       unsigned int n = fDim;
@@ -288,7 +288,7 @@ public:
    }
 
 private:
-   double DoEval(const double *x) const
+   double DoEval(const double *x) const override
    {
 
       gNCall++;
@@ -300,7 +300,7 @@ private:
       return f;
    }
 
-   double DoDerivative(const double *x, unsigned int i) const
+   double DoDerivative(const double *x, unsigned int i) const override
    {
       std::vector<double> g(fDim);
       Gradient(x, &g[0]);
@@ -396,14 +396,14 @@ const double *TrueMinimum(const ROOT::Math::IMultiGenFunction &func)
 {
 
    const RosenBrockFunction *fRB = dynamic_cast<const RosenBrockFunction *>(&func);
-   if (fRB != 0)
+   if (fRB != nullptr)
       return fRB->TrueMinimum();
    const TrigoFletcherFunction *fTF = dynamic_cast<const TrigoFletcherFunction *>(&func);
-   if (fTF != 0)
+   if (fTF != nullptr)
       return fTF->TrueMinimum();
    //    const ChebyQuadFunction * fCQ = dynamic_cast< const ChebyQuadFunction *> (&func);
    //    if (fCQ != 0) return fCQ->TrueMinimum();
-   return 0;
+   return nullptr;
 }
 
 void printMinimum(const std::vector<double> &x)
@@ -439,7 +439,7 @@ int DoNewMinimization(const ROOT::Math::IMultiGenFunction &func, const double *x
    min->SetPrintLevel(gVerbose);
    // check if func provides gradient
    const ROOT::Math::IMultiGradFunction *gfunc = dynamic_cast<const ROOT::Math::IMultiGradFunction *>(&func);
-   if (gfunc != 0 && useGradient)
+   if (gfunc != nullptr && useGradient)
       min->SetFunction(*gfunc);
    else
       min->SetFunction(func);
@@ -463,7 +463,7 @@ int DoNewMinimization(const ROOT::Math::IMultiGenFunction &func, const double *x
 
    bool ok = true;
    const double *trueMin = TrueMinimum(func);
-   if (trueMin != 0) {
+   if (trueMin != nullptr) {
       for (unsigned int i = 0; i < func.NDim(); ++i)
          ok &= (std::fabs(xmin[i] - trueMin[i]) < gAbsTolerance);
    }
@@ -501,7 +501,7 @@ int DoOldMinimization(FCN func, TVirtualFitter *min, double &minval, double &edm
 
    int iret = 0;
 
-   assert(min != 0);
+   assert(min != nullptr);
    min->SetFCN(func);
 
    Double_t arglist[100];
@@ -521,17 +521,17 @@ int DoOldMinimization(FCN func, TVirtualFitter *min, double &minval, double &edm
    if (minos)
       min->ExecuteCommand("MINOS", arglist, 0);
 
-   Double_t parx, pary;
+   Double_t parX, parY;
    Double_t we, al, bl;
    Char_t parName[32];
-   min->GetParameter(0, parName, parx, we, al, bl);
-   min->GetParameter(1, parName, pary, we, al, bl);
+   min->GetParameter(0, parName, parX, we, al, bl);
+   min->GetParameter(1, parName, parY, we, al, bl);
 
-   bool ok = (TMath::Abs(parx - 1.) < gAbsTolerance && TMath::Abs(pary - 1.) < gAbsTolerance);
+   bool ok = (TMath::Abs(parX - 1.) < gAbsTolerance && TMath::Abs(parY - 1.) < gAbsTolerance);
 
    double errdef = 0;
-   int nvpar, nparx;
-   min->GetStats(minval, edm, errdef, nvpar, nparx);
+   int nvpar, nparX;
+   min->GetStats(minval, edm, errdef, nvpar, nparX);
    if (!ok)
       iret = -2;
 
@@ -555,7 +555,7 @@ int testNewMinimizer(const ROOT::Math::IMultiGenFunction &func, const double *x0
    w.Start();
 
    ROOT::Math::Minimizer *min = ROOT::Math::Factory::CreateMinimizer(minimizer, algoType);
-   if (min == 0) {
+   if (min == nullptr) {
       std::cout << "Error using minimizer " << minimizer << "  " << algoType << std::endl;
       return -1;
    }
@@ -609,7 +609,7 @@ int testOldMinimizer(FCN func, const std::string &fitter, int n = 25)
 
    TVirtualFitter::SetDefaultFitter(fitter.c_str());
 
-   TVirtualFitter *min = TVirtualFitter::Fitter(0, n);
+   TVirtualFitter *min = TVirtualFitter::Fitter(nullptr, n);
 
    // min->Dump();
 
