@@ -19,7 +19,7 @@
 \class RooCurve
 \ingroup Roofitcore
 
-A RooCurve is a one-dimensional graphical representation of a real-valued function.
+One-dimensional graphical representation of a real-valued function.
 A curve is approximated by straight line segments with end points chosen to give
 a "good" approximation to the true curve. The goodness of the approximation is
 controlled by a precision and a resolution parameter.
@@ -41,7 +41,6 @@ To retrieve a RooCurve from a RooPlot, use RooPlot::getCurve().
 #include "RooAbsReal.h"
 #include "RooArgSet.h"
 #include "RooRealVar.h"
-#include "RooRealIntegral.h"
 #include "RooRealBinding.h"
 #include "RooMsgService.h"
 #include "RooProduct.h"
@@ -57,10 +56,27 @@ To retrieve a RooCurve from a RooPlot, use RooPlot::getCurve().
 #include <deque>
 #include <algorithm>
 
-using namespace std ;
+using std::endl, std::ostream, std::list, std::vector, std::cout, std::min;
 
 ClassImp(RooCurve);
 
+namespace {
+
+// Helpers to manage points
+struct Point {
+   double x;
+   double y;
+};
+
+inline Point getPoint(TGraph const &gr, int i)
+{
+   Point p;
+   gr.GetPoint(i, p.x, p.y);
+   return p;
+}
+
+} // namespace
+
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Default constructor.
@@ -79,15 +95,11 @@ RooCurve::RooCurve()
 /// factor to rescale the expression after normalization.
 /// If shiftToZero is set, the entire curve is shifted down to make the lowest
 /// point of the curve go through zero.
-RooCurve::RooCurve(const RooAbsReal &f, RooAbsRealLValue &x, double xlo, double xhi, Int_t xbins,
-    double scaleFactor, const RooArgSet *normVars, double prec, double resolution,
-    bool shiftToZero, WingMode wmode, Int_t nEvalError, Int_t doEEVal, double eeVal,
-    bool showProg) :
-        TGraph(),
-        RooPlotable(),
-        _showProgress(showProg)
+RooCurve::RooCurve(const RooAbsReal &f, RooAbsRealLValue &x, double xlo, double xhi, Int_t xbins, double scaleFactor,
+                   const RooArgSet *normVars, double prec, double resolution, bool shiftToZero, WingMode wmode,
+                   Int_t nEvalError, Int_t doEEVal, double eeVal, bool showProg)
+   : _showProgress(showProg)
 {
-
   // grab the function's name and title
   TString name(f.GetName());
   SetName(name.Data());
@@ -188,25 +200,22 @@ RooCurve::RooCurve(const char* name, const char* title, const RooCurve& c1, cons
   SetTitle(title) ;
 
   // Make deque of points in X
-  deque<double> pointList ;
-  double x,y ;
+  std::deque<double> pointList ;
 
   // Add X points of C1
-  Int_t i1,n1 = c1.GetN() ;
-  for (i1=0 ; i1<n1 ; i1++) {
-    c1.GetPoint(i1,x,y) ;
-    pointList.push_back(x) ;
+  Int_t n1 = c1.GetN();
+  for (int i1=0 ; i1<n1 ; i1++) {
+    pointList.push_back(c1.GetPointX(i1));
   }
 
   // Add X points of C2
-  Int_t i2,n2 = c2.GetN() ;
-  for (i2=0 ; i2<n2 ; i2++) {
-    c2.GetPoint(i2,x,y) ;
-    pointList.push_back(x) ;
+  Int_t n2 = c2.GetN();
+  for (int i2=0 ; i2<n2 ; i2++) {
+    pointList.push_back(c2.GetPointX(i2));
   }
 
   // Sort X points
-  sort(pointList.begin(),pointList.end()) ;
+  std::sort(pointList.begin(),pointList.end()) ;
 
   // Loop over X points
   double last(-RooNumber::infinity()) ;
@@ -250,19 +259,15 @@ void RooCurve::shiftCurveToZero()
 
    // First iteration, find current lowest point
    for (int i = 1; i < GetN() - 1; i++) {
-      double x;
-      double y;
-      GetPoint(i, x, y);
+      double y = GetPointY(i);
       minVal = std::min(y, minVal);
       maxVal = std::max(y, maxVal);
    }
 
    // Second iteration, lower all points by minVal
    for (int i = 1; i < GetN() - 1; i++) {
-      double x;
-      double y;
-      GetPoint(i, x, y);
-      SetPoint(i, x, y - minVal);
+      Point point = getPoint(*this, i);
+      SetPoint(i, point.x, point.y - minVal);
    }
 
    setYAxisLimits(0, maxVal - minVal);
@@ -342,7 +347,8 @@ void RooCurve::addPoints(const RooAbsFunc &func, double xlo, double xhi,
 
   // store points of the coarse scan and calculate any refinements necessary
   double minDx= resolution*(xhi-xlo);
-  double x1,x2= xlo;
+  double x1;
+  double x2 = xlo;
 
   if (wmode==Extended) {
     // Add two points to make curve jump from 0 to yval at the left end of the plotting range.
@@ -515,7 +521,7 @@ void RooCurve::printMultiline(ostream& os, Int_t /*contents*/, bool /*verbose*/,
   os << indent << "  Contains " << n << " points" << endl;
   os << indent << "  Graph points:" << endl;
   for(Int_t i= 0; i < n; i++) {
-    os << indent << setw(3) << i << ") x = " << fX[i] << " , y = " << fY[i] << endl;
+    os << indent << std::setw(3) << i << ") x = " << fX[i] << " , y = " << fY[i] << endl;
   }
 }
 
@@ -528,37 +534,34 @@ void RooCurve::printMultiline(ostream& os, Int_t /*contents*/, bool /*verbose*/,
 
 double RooCurve::chiSquare(const RooHist& hist, Int_t nFitParam) const
 {
-  Int_t i,np = hist.GetN() ;
-  double x,y,eyl,eyh,exl,exh ;
+  Int_t np = hist.GetN();
 
   // Find starting and ending bin of histogram based on range of RooCurve
-  double xstart,xstop ;
-
-  GetPoint(0,xstart,y) ;
-  GetPoint(GetN()-1,xstop,y) ;
+  double xstart = GetPointX(0);
+  double xstop = GetPointX(GetN()-1);
 
   Int_t nbin(0) ;
 
   ROOT::Math::KahanSum<double> chisq;
-  for (i=0 ; i<np ; i++) {
+  for (int i=0 ; i<np ; i++) {
 
     // Retrieve histogram contents
-    hist.GetPoint(i,x,y) ;
+    Point point = getPoint(hist, i);
 
     // Check if point is in range of curve
-    if (x<xstart || x>xstop) continue ;
+    if (point.x<xstart || point.x>xstop) continue ;
 
-    eyl = hist.GetEYlow()[i] ;
-    eyh = hist.GetEYhigh()[i] ;
-    exl = hist.GetEXlow()[i] ;
-    exh = hist.GetEXhigh()[i] ;
+    double eyl = hist.GetEYlow()[i] ;
+    double eyh = hist.GetEYhigh()[i] ;
+    double exl = hist.GetEXlow()[i] ;
+    double exh = hist.GetEXhigh()[i] ;
 
     // Integrate function over this bin
-    double avg = average(x-exl,x+exh) ;
+    double avg = average(point.x-exl,point.x+exh) ;
 
     // Add pull^2 to chisq
-    if (y!=0) {
-      double pull = (y>avg) ? ((y-avg)/eyl) : ((y-avg)/eyh) ;
+    if (point.y!=0) {
+      double pull = (point.y>avg) ? ((point.y-avg)/eyl) : ((point.y-avg)/eyh) ;
       chisq += pull*pull ;
       nbin++ ;
     }
@@ -587,48 +590,37 @@ double RooCurve::average(double xFirst, double xLast) const
   double yLast = interpolate(xLast,1e-10) ;
 
   // Find first and last mid points
-  Int_t ifirst = findPoint(xFirst,1e10) ;
-  Int_t ilast  = findPoint(xLast,1e10) ;
-  double xFirstPt,yFirstPt,xLastPt,yLastPt ;
-  GetPoint(ifirst,xFirstPt,yFirstPt) ;
-  GetPoint(ilast,xLastPt,yLastPt) ;
+  Int_t ifirst = findPoint(xFirst, std::numeric_limits<double>::infinity());
+  Int_t ilast  = findPoint(xLast, std::numeric_limits<double>::infinity());
 
-  double tolerance=1e-3*(xLast-xFirst) ;
+  // Make sure the midpoints are actually in the interval
+  while (GetPointX(ifirst) < xFirst) {
+    ++ifirst;
+  }
+  while (GetPointX(ilast) > xLast) {
+    --ilast;
+  }
 
   // Handle trivial scenario -- no midway points, point only at or outside given range
-  if (ilast-ifirst==1 &&(xFirstPt-xFirst)<-1*tolerance && (xLastPt-xLast)>tolerance) {
+  if (ilast < ifirst) {
     return 0.5*(yFirst+yLast) ;
   }
 
-  // If first point closest to xFirst is at xFirst or before xFirst take the next point
-  // as the first midway point
-  if ((xFirstPt-xFirst)<-1*tolerance) {
-    ifirst++ ;
-    const_cast<RooCurve&>(*this).GetPoint(ifirst,xFirstPt,yFirstPt) ;
-  }
-
-  // If last point closest to yLast is at yLast or beyond yLast the previous point
-  // as the last midway point
-  if ((xLastPt-xLast)>tolerance) {
-    ilast-- ;
-    const_cast<RooCurve&>(*this).GetPoint(ilast,xLastPt,yLastPt) ;
-  }
-
-  double sum(0),x1,y1,x2,y2 ;
+  Point firstPt = getPoint(*this, ifirst);
+  Point lastPt = getPoint(*this, ilast);
 
   // Trapezoid integration from lower edge to first midpoint
-  sum += (xFirstPt-xFirst)*(yFirst+yFirstPt)/2 ;
+  double sum = 0.5 * (firstPt.x-xFirst)*(yFirst+firstPt.y);
 
   // Trapezoid integration between midpoints
-  Int_t i ;
-  for (i=ifirst ; i<ilast ; i++) {
-    const_cast<RooCurve&>(*this).GetPoint(i,x1,y1) ;
-    const_cast<RooCurve&>(*this).GetPoint(i+1,x2,y2) ;
-    sum += (x2-x1)*(y1+y2)/2 ;
+  for (int i=ifirst ; i<ilast ; i++) {
+    Point p1 = getPoint(*this, i) ;
+    Point p2 = getPoint(*this, i+1) ;
+    sum += 0.5 * (p2.x-p1.x)*(p1.y+p2.y);
   }
 
   // Trapezoid integration from last midpoint to upper edge
-  sum += (xLast-xLastPt)*(yLastPt+yLast)/2 ;
+  sum += 0.5 * (xLast-lastPt.x)*(lastPt.y+yLast);
   return sum/(xLast-xFirst) ;
 }
 
@@ -640,11 +632,11 @@ double RooCurve::average(double xFirst, double xLast) const
 
 Int_t RooCurve::findPoint(double xvalue, double tolerance) const
 {
-  double delta(std::numeric_limits<double>::max()),x,y ;
-  Int_t i,n = GetN() ;
+  double delta(std::numeric_limits<double>::max());
+  Int_t n = GetN();
   Int_t ibest(-1) ;
-  for (i=0 ; i<n ; i++) {
-    GetPoint(i,x,y);
+  for (int i=0 ; i<n ; i++) {
+    double x = GetPointX(i);
     if (std::abs(xvalue-x)<delta) {
       delta = std::abs(xvalue-x) ;
       ibest = i ;
@@ -667,33 +659,32 @@ double RooCurve::interpolate(double xvalue, double tolerance) const
   int ibest = findPoint(xvalue,1e10) ;
 
   // Get position of best point
-  double xbest, ybest ;
-  const_cast<RooCurve*>(this)->GetPoint(ibest,xbest,ybest) ;
+  Point pbest = getPoint(*this, ibest);
 
   // Handle trivial case of being dead on
-  if (std::abs(xbest-xvalue)<tolerance) {
-    return ybest ;
+  if (std::abs(pbest.x-xvalue)<tolerance) {
+    return pbest.y;
   }
 
   // Get nearest point on other side w.r.t. xvalue
-  double xother,yother, retVal(0) ;
-  if (xbest<xvalue) {
+  double retVal(0);
+  if (pbest.x<xvalue) {
     if (ibest==n-1) {
       // Value beyond end requested -- return value of last point
-      return ybest ;
+      return pbest.y ;
     }
-    const_cast<RooCurve*>(this)->GetPoint(ibest+1,xother,yother) ;
-    if (xother==xbest) return ybest ;
-    retVal = ybest + (yother-ybest)*(xvalue-xbest)/(xother-xbest) ;
+    Point pother = getPoint(*this, ibest+1);
+    if (pother.x==pbest.x) return pbest.y ;
+    retVal = pbest.y + (pother.y-pbest.y)*(xvalue-pbest.x)/(pother.x-pbest.x) ;
 
   } else {
     if (ibest==0) {
       // Value before 1st point requested -- return value of 1st point
-      return ybest ;
+      return pbest.y ;
     }
-    const_cast<RooCurve*>(this)->GetPoint(ibest-1,xother,yother) ;
-    if (xother==xbest) return ybest ;
-    retVal = yother + (ybest-yother)*(xvalue-xother)/(xbest-xother) ;
+    Point pother = getPoint(*this, ibest-1);
+    if (pother.x==pbest.x) return pbest.y ;
+    retVal = pother.y + (pbest.y-pother.y)*(xvalue-pother.x)/(pbest.x-pother.x) ;
   }
 
   return retVal ;
@@ -788,7 +779,8 @@ RooCurve* RooCurve::makeErrorBand(const vector<RooCurve*>& plusVar, const vector
 
 void RooCurve::calcBandInterval(const vector<RooCurve*>& plusVar, const vector<RooCurve*>& minusVar,Int_t i, const TMatrixD& C, double /*Z*/, double& lo, double& hi) const
 {
-  vector<double> y_plus(plusVar.size()), y_minus(minusVar.size()) ;
+  vector<double> y_plus(plusVar.size());
+  vector<double> y_minus(minusVar.size());
   Int_t j(0) ;
   for (vector<RooCurve*>::const_iterator iter=plusVar.begin() ; iter!=plusVar.end() ; ++iter) {
     y_plus[j++] = (*iter)->interpolate(GetX()[i]) ;
@@ -834,7 +826,8 @@ void RooCurve::calcBandInterval(const vector<RooCurve*>& variations,Int_t i,doub
     hi = y[y.size()-delta] ;
   } else {
     // Estimate R.M.S of variations at each point and use that as Gaussian sigma
-    double sum_y(0), sum_ysq(0) ;
+    double sum_y(0);
+    double sum_ysq(0);
     for (unsigned int k=0 ; k<y.size() ; k++) {
       sum_y   += y[k] ;
       sum_ysq += y[k]*y[k] ;
@@ -859,7 +852,10 @@ bool RooCurve::isIdentical(const RooCurve& other, double tol, bool verbose) cons
 {
   // Determine X range and Y range
   Int_t n= min(GetN(),other.GetN());
-  double xmin(1e30), xmax(-1e30), ymin(1e30), ymax(-1e30) ;
+  double xmin(1e30);
+  double xmax(-1e30);
+  double ymin(1e30);
+  double ymax(-1e30);
   for(Int_t i= 0; i < n; i++) {
     if (fX[i]<xmin) xmin=fX[i] ;
     if (fX[i]>xmax) xmax=fX[i] ;
