@@ -58,7 +58,7 @@
 #include "RooChi2Var.h"
 #endif
 
-using namespace std;
+using std::cout,std::endl, std::ostream, std::istream, std::list;
 
 ClassImp(RooFormulaVar);
 
@@ -161,9 +161,9 @@ double RooFormulaVar::evaluate() const
 }
 
 
-void RooFormulaVar::computeBatch(double* output, size_t nEvents, RooFit::Detail::DataMap const& dataMap) const
+void RooFormulaVar::doEval(RooFit::EvalContext &ctx) const
 {
-  getFormula().computeBatch(output, nEvents, dataMap);
+   getFormula().doEval(ctx);
 }
 
 
@@ -286,10 +286,10 @@ double RooFormulaVar::defaultErrorLevel() const
 #ifdef ROOFIT_LEGACY_EVAL_BACKEND
   for (const auto arg : _actualVars) {
     if (dynamic_cast<RooNLLVar*>(arg)) {
-      nllArg = (RooAbsReal*)arg ;
+      nllArg = static_cast<RooAbsReal*>(arg) ;
     }
     if (dynamic_cast<RooChi2Var*>(arg)) {
-      chi2Arg = (RooAbsReal*)arg ;
+      chi2Arg = static_cast<RooAbsReal*>(arg) ;
     }
   }
 #endif
@@ -313,6 +313,8 @@ double RooFormulaVar::defaultErrorLevel() const
   return 1.0 ;
 }
 
-
-
-
+void RooFormulaVar::translate(RooFit::Detail::CodeSquashContext &ctx) const
+{
+   getVal(); // to trigger the creation of the TFormula
+   ctx.addResult(this, ctx.buildCall(_formula->getTFormula()->GetUniqueFuncName().Data(), _actualVars));
+}
