@@ -19,7 +19,7 @@
 \class RooRealMPFE
 \ingroup Roofitcore
 
-RooRealMPFE is the multi-processor front-end for parallel calculation
+Multi-processor front-end for parallel calculation
 of RooAbsReal objects. Each RooRealMPFE forks a process that calculates
 the value of the proxies RooAbsReal object. The (re)calculation of
 the proxied object is started asynchronously with the calculate() option.
@@ -32,12 +32,12 @@ Simple use demonstration
 ~~~{.cpp}
 RooAbsReal* slowFunc ;
 
-Double_t val = slowFunc->getVal() // Evaluate slowFunc in current process
+double val = slowFunc->getVal() // Evaluate slowFunc in current process
 
 RooRealMPFE mpfe("mpfe","frontend to slowFunc",*slowFunc) ;
 mpfe.calculate() ;           // Start calculation of slow-func in remote process
                              // .. do other stuff here ..
-Double_t val = mpfe.getVal() // Wait for remote calculation to finish and retrieve value
+double val = mpfe.getVal() // Wait for remote calculation to finish and retrieve value
 ~~~
 
 For general multiprocessing in ROOT, please refer to the TProcessExecutor class.
@@ -45,7 +45,6 @@ For general multiprocessing in ROOT, please refer to the TProcessExecutor class.
 **/
 
 #include "Riostream.h"
-#include "RooFit.h"
 
 #ifndef _WIN32
 #include "BidirMMapPipe.h"
@@ -58,20 +57,42 @@ For general multiprocessing in ROOT, please refer to the TProcessExecutor class.
 #include "RooAbsCategory.h"
 #include "RooRealVar.h"
 #include "RooCategory.h"
-#include "RooMPSentinel.h"
 #include "RooMsgService.h"
 #include "RooNLLVar.h"
 #include "RooTrace.h"
 
+#include "Rtypes.h"
 #include "TSystem.h"
 
-RooMPSentinel RooRealMPFE::_sentinel ;
 
-using namespace std;
+class RooRealMPFE ;
+
+// RooMPSentinel is a singleton class that keeps track of all
+// parallel execution processes for goodness-of-fit calculations.
+// The primary task of RooMPSentinel is to terminate all server processes
+// when the main ROOT process is exiting.
+struct RooMPSentinel {
+
+  static RooMPSentinel& instance();
+
+  ~RooMPSentinel();
+
+  void add(RooRealMPFE& mpfe) ;
+  void remove(RooRealMPFE& mpfe) ;
+
+  RooArgSet _mpfeSet ;
+};
+
+RooMPSentinel& RooMPSentinel::instance() {
+  static RooMPSentinel inst;
+  return inst;
+}
+
+
+using std::cout, std::endl, std::string, std::ostringstream, std::list, std::pair;
 using namespace RooFit;
 
 ClassImp(RooRealMPFE);
-  ;
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -79,25 +100,25 @@ ClassImp(RooRealMPFE);
 /// asynchronously in a separate process. If calcInline is true the value of 'arg'
 /// is calculate synchronously in the current process.
 
-RooRealMPFE::RooRealMPFE(const char *name, const char *title, RooAbsReal& arg, Bool_t calcInline) :
+RooRealMPFE::RooRealMPFE(const char *name, const char *title, RooAbsReal& arg, bool calcInline) :
   RooAbsReal(name,title),
   _state(Initialize),
   _arg("arg","arg",this,arg),
   _vars("vars","vars",this),
-  _calcInProgress(kFALSE),
-  _verboseClient(kFALSE),
-  _verboseServer(kFALSE),
+  _calcInProgress(false),
+  _verboseClient(false),
+  _verboseServer(false),
   _inlineMode(calcInline),
   _remoteEvalErrorLoggingState(RooAbsReal::PrintErrors),
-  _pipe(0),
-  _updateMaster(0),
-  _retrieveDispatched(kFALSE), _evalCarry(0.)
+  _pipe(nullptr),
+  _updateMaster(nullptr),
+  _retrieveDispatched(false), _evalCarry(0.)
 {
 #ifdef _WIN32
-  _inlineMode = kTRUE;
+  _inlineMode = true;
 #endif
   initVars() ;
-  _sentinel.add(*this) ;
+  RooMPSentinel::instance().add(*this) ;
 
 }
 
@@ -112,18 +133,18 @@ RooRealMPFE::RooRealMPFE(const RooRealMPFE& other, const char* name) :
   _state(Initialize),
   _arg("arg",this,other._arg),
   _vars("vars",this,other._vars),
-  _calcInProgress(kFALSE),
+  _calcInProgress(false),
   _verboseClient(other._verboseClient),
   _verboseServer(other._verboseServer),
   _inlineMode(other._inlineMode),
   _forceCalc(other._forceCalc),
   _remoteEvalErrorLoggingState(other._remoteEvalErrorLoggingState),
-  _pipe(0),
-  _updateMaster(0),
-  _retrieveDispatched(kFALSE), _evalCarry(other._evalCarry)
+  _pipe(nullptr),
+  _updateMaster(nullptr),
+  _retrieveDispatched(false), _evalCarry(other._evalCarry)
 {
   initVars() ;
-  _sentinel.add(*this) ;
+  RooMPSentinel::instance().add(*this) ;
 }
 
 
@@ -134,7 +155,7 @@ RooRealMPFE::RooRealMPFE(const RooRealMPFE& other, const char* name) :
 RooRealMPFE::~RooRealMPFE()
 {
   if (_state==Client) standby();
-  _sentinel.remove(*this);
+  RooMPSentinel::instance().remove(*this);
 }
 
 
@@ -149,24 +170,21 @@ void RooRealMPFE::initVars()
   _saveVars.removeAll() ;
 
   // Retrieve non-constant parameters
-  RooArgSet* vars = _arg.arg().getParameters(RooArgSet()) ;
-  //RooArgSet* ncVars = (RooArgSet*) vars->selectByAttrib("Constant",kFALSE) ;
+  auto vars = _arg->getParameters(RooArgSet());
+  //RooArgSet* ncVars = (RooArgSet*) vars->selectByAttrib("Constant",false) ;
   RooArgList varList(*vars) ;
 
   // Save in lists
   _vars.add(varList) ;
   _saveVars.addClone(varList) ;
-  _valueChanged.resize(_vars.getSize()) ;
-  _constChanged.resize(_vars.getSize()) ;
+  _valueChanged.resize(_vars.size()) ;
+  _constChanged.resize(_vars.size()) ;
 
   // Force next calculation
-  _forceCalc = kTRUE ;
-
-  delete vars ;
-  //delete ncVars ;
+  _forceCalc = true ;
 }
 
-Double_t RooRealMPFE::getCarry() const
+double RooRealMPFE::getCarry() const
 {
   if (_inlineMode) {
     RooAbsTestStatistic* tmp = dynamic_cast<RooAbsTestStatistic*>(_arg.absArg());
@@ -204,18 +222,19 @@ void RooRealMPFE::initialize()
 
     // Kill server at end of service
     if (_verboseServer) ccoutD(Minimization) << "RooRealMPFE::initialize(" <<
-	GetName() << ") server process terminating" << endl ;
+   GetName() << ") server process terminating" << endl ;
 
     delete _arg.absArg();
     delete _pipe;
     _exit(0) ;
   } else {
-    // Client process - fork successul
-    if (_verboseClient) ccoutD(Minimization) << "RooRealMPFE::initialize(" <<
-	GetName() << ") successfully forked server process " <<
-	    _pipe->pidOtherEnd() << endl ;
+    // Client process - fork successful
+    if (_verboseClient) {
+       ccoutD(Minimization) << "RooRealMPFE::initialize(" << GetName() << ") successfully forked server process "
+                            << _pipe->pidOtherEnd() << endl;
+    }
     _state = Client ;
-    _calcInProgress = kFALSE ;
+    _calcInProgress = false ;
   }
 #endif // _WIN32
 }
@@ -231,9 +250,11 @@ void RooRealMPFE::serverLoop()
 #ifndef _WIN32
   int msg ;
 
-  Int_t idx, index, numErrors ;
-  Double_t value ;
-  Bool_t isConst ;
+  Int_t idx;
+  Int_t index;
+  Int_t numErrors;
+  double value ;
+  bool isConst ;
 
   clearEvalErrorLog() ;
 
@@ -241,7 +262,7 @@ void RooRealMPFE::serverLoop()
     *_pipe >> msg;
     if (Terminate == msg) {
       if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-			       << ") IPC fromClient> Terminate" << endl;
+                << ") IPC fromClient> Terminate" << endl;
       // send terminate acknowledged to client
       *_pipe << msg << BidirMMapPipe::flush;
       break;
@@ -250,100 +271,100 @@ void RooRealMPFE::serverLoop()
     switch (msg) {
     case SendReal:
       {
-	*_pipe >> idx >> value >> isConst;
-	if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-				 << ") IPC fromClient> SendReal [" << idx << "]=" << value << endl ;
-	RooRealVar* rvar = (RooRealVar*)_vars.at(idx) ;
-	rvar->setVal(value) ;
-	if (rvar->isConstant() != isConst) {
-	  rvar->setConstant(isConst) ;
-	}
+   *_pipe >> idx >> value >> isConst;
+   if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
+             << ") IPC fromClient> SendReal [" << idx << "]=" << value << endl ;
+   RooRealVar* rvar = static_cast<RooRealVar*>(_vars.at(idx)) ;
+   rvar->setVal(value) ;
+   if (rvar->isConstant() != isConst) {
+     rvar->setConstant(isConst) ;
+   }
       }
       break ;
 
     case SendCat:
       {
-	*_pipe >> idx >> index;
-	if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-				 << ") IPC fromClient> SendCat [" << idx << "]=" << index << endl ;
-	((RooCategory*)_vars.at(idx))->setIndex(index) ;
+   *_pipe >> idx >> index;
+   if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
+             << ") IPC fromClient> SendCat [" << idx << "]=" << index << endl ;
+   (static_cast<RooCategory*>(_vars.at(idx)))->setIndex(index) ;
       }
       break ;
 
     case Calculate:
       if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-			       << ") IPC fromClient> Calculate" << endl ;
+                << ") IPC fromClient> Calculate" << endl ;
       _value = _arg ;
       break ;
 
     case CalculateNoOffset:
       if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-			       << ") IPC fromClient> Calculate" << endl ;
+                << ") IPC fromClient> Calculate" << endl ;
 
-      RooAbsReal::setHideOffset(kFALSE) ;
+      RooAbsReal::setHideOffset(false) ;
       _value = _arg ;
-      RooAbsReal::setHideOffset(kTRUE) ;
+      RooAbsReal::setHideOffset(true) ;
       break ;
 
     case Retrieve:
       {
-	if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-				 << ") IPC fromClient> Retrieve" << endl ;
-	msg = ReturnValue;
-	numErrors = numEvalErrors();
-	*_pipe << msg << _value << getCarry() << numErrors;
+   if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
+             << ") IPC fromClient> Retrieve" << endl ;
+   msg = ReturnValue;
+   numErrors = numEvalErrors();
+   *_pipe << msg << _value << getCarry() << numErrors;
 
-	if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-				 << ") IPC toClient> ReturnValue " << _value << " NumError " << numErrors << endl ;
+   if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
+             << ") IPC toClient> ReturnValue " << _value << " NumError " << numErrors << endl ;
 
-	if (numErrors) {
-	  // Loop over errors
-	  std::string objidstr;
-	  {
-	    ostringstream oss2;
-	    // Format string with object identity as this cannot be evaluated on the other side
-	    oss2 << "PID" << gSystem->GetPid() << "/";
-	    printStream(oss2,kName|kClassName|kArgs,kInline);
-	    objidstr = oss2.str();
-	  }
-	  std::map<const RooAbsArg*,pair<string,list<EvalError> > >::const_iterator iter = evalErrorIter();
-	  const RooAbsArg* ptr = 0;
-	  for (int i = 0; i < numEvalErrorItems(); ++i) {
-	    list<EvalError>::const_iterator iter2 = iter->second.second.begin();
-	    for (; iter->second.second.end() != iter2; ++iter2) {
-	      ptr = iter->first;
-	      *_pipe << ptr << iter2->_msg << iter2->_srvval << objidstr;
-	      if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-		<< ") IPC toClient> sending error log Arg " << iter->first << " Msg " << iter2->_msg << endl ;
-	    }
-	  }
-	  // let other end know that we're done with the list of errors
-	  ptr = 0;
-	  *_pipe << ptr;
-	  // Clear error list on local side
-	  clearEvalErrorLog();
-	}
-	*_pipe << BidirMMapPipe::flush;
+   if (numErrors) {
+     // Loop over errors
+     std::string objidstr;
+     {
+       ostringstream oss2;
+       // Format string with object identity as this cannot be evaluated on the other side
+       oss2 << "PID" << gSystem->GetPid() << "/";
+       printStream(oss2,kName|kClassName|kArgs,kInline);
+       objidstr = oss2.str();
+     }
+     std::map<const RooAbsArg*,pair<string,list<EvalError> > >::const_iterator iter = evalErrorIter();
+     const RooAbsArg* ptr = nullptr;
+     for (int i = 0; i < numEvalErrorItems(); ++i) {
+       list<EvalError>::const_iterator iter2 = iter->second.second.begin();
+       for (; iter->second.second.end() != iter2; ++iter2) {
+         ptr = iter->first;
+         *_pipe << ptr << iter2->_msg << iter2->_srvval << objidstr;
+         if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
+      << ") IPC toClient> sending error log Arg " << iter->first << " Msg " << iter2->_msg << endl ;
+       }
+     }
+     // let other end know that we're done with the list of errors
+     ptr = nullptr;
+     *_pipe << ptr;
+     // Clear error list on local side
+     clearEvalErrorLog();
+   }
+   *_pipe << BidirMMapPipe::flush;
       }
       break;
 
     case ConstOpt:
       {
-	Bool_t doTrack ;
-	int code;
-	*_pipe >> code >> doTrack;
-	if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-				 << ") IPC fromClient> ConstOpt " << code << " doTrack = " << (doTrack?"T":"F") << endl ;
-	((RooAbsReal&)_arg.arg()).constOptimizeTestStatistic(static_cast<RooAbsArg::ConstOpCode>(code),doTrack) ;
-	break ;
+   bool doTrack ;
+   int code;
+   *_pipe >> code >> doTrack;
+   if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
+             << ") IPC fromClient> ConstOpt " << code << " doTrack = " << (doTrack?"T":"F") << endl ;
+   ((RooAbsReal&)_arg.arg()).constOptimizeTestStatistic(static_cast<RooAbsArg::ConstOpCode>(code),doTrack) ;
+   break ;
       }
 
     case Verbose:
       {
-      Bool_t flag ;
+      bool flag ;
       *_pipe >> flag;
       if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-			       << ") IPC fromClient> Verbose " << (flag?1:0) << endl ;
+                << ") IPC fromClient> Verbose " << (flag?1:0) << endl ;
       _verboseServer = flag ;
       }
       break ;
@@ -351,10 +372,10 @@ void RooRealMPFE::serverLoop()
 
     case ApplyNLLW2:
       {
-      Bool_t flag ;
+      bool flag ;
       *_pipe >> flag;
       if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-			       << ") IPC fromClient> ApplyNLLW2 " << (flag?1:0) << endl ;
+                << ") IPC fromClient> ApplyNLLW2 " << (flag?1:0) << endl ;
 
       // Do application of weight-squared here
       doApplyNLLW2(flag) ;
@@ -363,10 +384,10 @@ void RooRealMPFE::serverLoop()
 
     case EnableOffset:
       {
-      Bool_t flag ;
+      bool flag ;
       *_pipe >> flag;
       if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-			       << ") IPC fromClient> EnableOffset " << (flag?1:0) << endl ;
+                << ") IPC fromClient> EnableOffset " << (flag?1:0) << endl ;
 
       // Enable likelihoof offsetting here
       ((RooAbsReal&)_arg.arg()).enableOffsetting(flag) ;
@@ -375,19 +396,19 @@ void RooRealMPFE::serverLoop()
 
     case LogEvalError:
       {
-	int iflag2;
-	*_pipe >> iflag2;
-	RooAbsReal::ErrorLoggingMode flag2 = static_cast<RooAbsReal::ErrorLoggingMode>(iflag2);
-	RooAbsReal::setEvalErrorLoggingMode(flag2) ;
-	if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-				 << ") IPC fromClient> LogEvalError flag = " << flag2 << endl ;
+   int iflag2;
+   *_pipe >> iflag2;
+   RooAbsReal::ErrorLoggingMode flag2 = static_cast<RooAbsReal::ErrorLoggingMode>(iflag2);
+   RooAbsReal::setEvalErrorLoggingMode(flag2) ;
+   if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
+             << ") IPC fromClient> LogEvalError flag = " << flag2 << endl ;
       }
       break ;
 
 
     default:
       if (_verboseServer) cout << "RooRealMPFE::serverLoop(" << GetName()
-			       << ") IPC fromClient> Unknown message (code = " << msg << ")" << endl ;
+                << ") IPC fromClient> Unknown message (code = " << msg << ")" << endl ;
       break ;
     }
   }
@@ -424,51 +445,52 @@ void RooRealMPFE::calculate() const
   if (_state==Client) {
     //     cout << "RooRealMPFE::calculate(" << GetName() << ") state is Client trigger remote calculation" << endl ;
     Int_t i(0) ;
-    RooFIter viter = _vars.fwdIterator() ;
-    RooFIter siter = _saveVars.fwdIterator() ;
 
-    //for (i=0 ; i<_vars.getSize() ; i++) {
-    RooAbsArg *var, *saveVar ;
-    while((var = viter.next())) {
-      saveVar = siter.next() ;
+    //for (i=0 ; i<_vars.size() ; i++) {
+    RooAbsArg *var;
+    RooAbsArg *saveVar;
+    for (std::size_t j=0 ; j<_vars.size() ; j++) {
+      var = _vars.at(j);
+      saveVar = _saveVars.at(j);
 
-      //Bool_t valChanged = !(*var==*saveVar) ;
-      Bool_t valChanged,constChanged  ;
+      //bool valChanged = !(*var==*saveVar) ;
+      bool valChanged;
+      bool constChanged;
       if (!_updateMaster) {
-	valChanged = !var->isIdentical(*saveVar,kTRUE) ;
-	constChanged = (var->isConstant() != saveVar->isConstant()) ;
-	_valueChanged[i] = valChanged ;
-	_constChanged[i] = constChanged ;
+   valChanged = !var->isIdentical(*saveVar,true) ;
+   constChanged = (var->isConstant() != saveVar->isConstant()) ;
+   _valueChanged[i] = valChanged ;
+   _constChanged[i] = constChanged ;
       } else {
-	valChanged = _updateMaster->_valueChanged[i] ;
-	constChanged = _updateMaster->_constChanged[i] ;
+   valChanged = _updateMaster->_valueChanged[i] ;
+   constChanged = _updateMaster->_constChanged[i] ;
       }
 
       if ( valChanged || constChanged || _forceCalc) {
-	//cout << "RooRealMPFE::calculate(" << GetName() << " variable " << var->GetName() << " changed " << endl ;
-	if (_verboseClient) cout << "RooRealMPFE::calculate(" << GetName()
-				 << ") variable " << _vars.at(i)->GetName() << " changed" << endl ;
-	if (constChanged) {
-	  ((RooRealVar*)saveVar)->setConstant(var->isConstant()) ;
-	}
-	saveVar->copyCache(var) ;
+   //cout << "RooRealMPFE::calculate(" << GetName() << " variable " << var->GetName() << " changed " << endl ;
+   if (_verboseClient) cout << "RooRealMPFE::calculate(" << GetName()
+             << ") variable " << _vars.at(i)->GetName() << " changed" << endl ;
+   if (constChanged) {
+     (static_cast<RooRealVar*>(saveVar))->setConstant(var->isConstant()) ;
+   }
+   saveVar->copyCache(var) ;
 
-	// send message to server
-	if (dynamic_cast<RooAbsReal*>(var)) {
-	  int msg = SendReal ;
-	  Double_t val = ((RooAbsReal*)var)->getVal() ;
-	  Bool_t isC = var->isConstant() ;
-	  *_pipe << msg << i << val << isC;
+   // send message to server
+   if (dynamic_cast<RooAbsReal*>(var)) {
+     int msg = SendReal ;
+     double val = (static_cast<RooAbsReal*>(var))->getVal() ;
+     bool isC = var->isConstant() ;
+     *_pipe << msg << i << val << isC;
 
-	  if (_verboseServer) cout << "RooRealMPFE::calculate(" << GetName()
-				   << ") IPC toServer> SendReal [" << i << "]=" << val << (isC?" (Constant)":"") <<  endl ;
-	} else if (dynamic_cast<RooAbsCategory*>(var)) {
-	  int msg = SendCat ;
-	  UInt_t idx = ((RooAbsCategory*)var)->getCurrentIndex() ;
-	  *_pipe << msg << i << idx;
-	  if (_verboseServer) cout << "RooRealMPFE::calculate(" << GetName()
-				   << ") IPC toServer> SendCat [" << i << "]=" << idx << endl ;
-	}
+     if (_verboseServer) cout << "RooRealMPFE::calculate(" << GetName()
+               << ") IPC toServer> SendReal [" << i << "]=" << val << (isC?" (Constant)":"") <<  endl ;
+   } else if (dynamic_cast<RooAbsCategory*>(var)) {
+     int msg = SendCat ;
+     UInt_t idx = (static_cast<RooAbsCategory*>(var))->getCurrentIndex() ;
+     *_pipe << msg << i << idx;
+     if (_verboseServer) cout << "RooRealMPFE::calculate(" << GetName()
+               << ") IPC toServer> SendCat [" << i << "]=" << idx << endl ;
+   }
       }
       i++ ;
     }
@@ -476,22 +498,22 @@ void RooRealMPFE::calculate() const
     int msg = hideOffset() ? Calculate : CalculateNoOffset;
     *_pipe << msg;
     if (_verboseServer) cout << "RooRealMPFE::calculate(" << GetName()
-			     << ") IPC toServer> Calculate " << endl ;
+              << ") IPC toServer> Calculate " << endl ;
 
     // Clear dirty state and mark that calculation request was dispatched
     clearValueDirty() ;
-    _calcInProgress = kTRUE ;
-    _forceCalc = kFALSE ;
+    _calcInProgress = true ;
+    _forceCalc = false ;
 
     msg = Retrieve ;
     *_pipe << msg << BidirMMapPipe::flush;
     if (_verboseServer) cout << "RooRealMPFE::evaluate(" << GetName()
-			     << ") IPC toServer> Retrieve " << endl ;
-    _retrieveDispatched = kTRUE ;
+              << ") IPC toServer> Retrieve " << endl ;
+    _retrieveDispatched = true ;
 
   } else if (_state!=Inline) {
     cout << "RooRealMPFE::calculate(" << GetName()
-	 << ") ERROR not in Client or Inline mode" << endl ;
+    << ") ERROR not in Client or Inline mode" << endl ;
   }
 
 
@@ -502,17 +524,17 @@ void RooRealMPFE::calculate() const
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// If value needs recalculation and calculation has not beed started
+/// If value needs recalculation and calculation has not been started
 /// with a call to calculate() start it now. This function blocks
 /// until remote process has finished calculation and returns
 /// remote value
 
-Double_t RooRealMPFE::getValV(const RooArgSet* /*nset*/) const
+double RooRealMPFE::getValV(const RooArgSet* /*nset*/) const
 {
 
   if (isValueDirty()) {
     // Cache is dirty, no calculation has been started yet
-    //cout << "RooRealMPFE::getValF(" << GetName() << ") cache is dirty, caling calculate and evaluate" << endl ;
+    //cout << "RooRealMPFE::getValF(" << GetName() << ") cache is dirty, calling calculate and evaluate" << endl ;
     calculate() ;
     _value = evaluate() ;
   } else if (_calcInProgress) {
@@ -535,19 +557,19 @@ Double_t RooRealMPFE::getValV(const RooArgSet* /*nset*/) const
 /// If error were logged use logEvalError() on remote side
 /// transfer those errors to the local eval error queue.
 
-Double_t RooRealMPFE::evaluate() const
+double RooRealMPFE::evaluate() const
 {
   // Retrieve value of arg
-  Double_t return_value = 0;
+  double return_value = 0;
   if (_state==Inline) {
     return_value = _arg ;
   } else if (_state==Client) {
 #ifndef _WIN32
     bool needflush = false;
     int msg;
-    Double_t value;
+    double value;
 
-    // If current error loggin state is not the same as remote state
+    // If current error logging state is not the same as remote state
     // update the remote state
     if (evalErrorLoggingMode() != _remoteEvalErrorLoggingState) {
       msg = LogEvalError ;
@@ -562,10 +584,10 @@ Double_t RooRealMPFE::evaluate() const
       *_pipe << msg;
       needflush = true;
       if (_verboseServer) cout << "RooRealMPFE::evaluate(" << GetName()
-			       << ") IPC toServer> Retrieve " << endl ;
+                << ") IPC toServer> Retrieve " << endl ;
     }
     if (needflush) *_pipe << BidirMMapPipe::flush;
-    _retrieveDispatched = kFALSE ;
+    _retrieveDispatched = false ;
 
 
     Int_t numError;
@@ -574,26 +596,28 @@ Double_t RooRealMPFE::evaluate() const
 
     if (msg!=ReturnValue) {
       cout << "RooRealMPFE::evaluate(" << GetName()
-	   << ") ERROR: unexpected message from server process: " << msg << endl ;
+      << ") ERROR: unexpected message from server process: " << msg << endl ;
       return 0 ;
     }
     if (_verboseServer) cout << "RooRealMPFE::evaluate(" << GetName()
-			     << ") IPC fromServer> ReturnValue " << value << endl ;
+              << ") IPC fromServer> ReturnValue " << value << endl ;
 
     if (_verboseServer) cout << "RooRealMPFE::evaluate(" << GetName()
-			     << ") IPC fromServer> NumErrors " << numError << endl ;
+              << ") IPC fromServer> NumErrors " << numError << endl ;
     if (numError) {
       // Retrieve remote errors and feed into local error queue
-      char *msgbuf1 = 0, *msgbuf2 = 0, *msgbuf3 = 0;
-      RooAbsArg *ptr = 0;
+      char *msgbuf1 = nullptr;
+      char *msgbuf2 = nullptr;
+      char *msgbuf3 = nullptr;
+      RooAbsArg *ptr = nullptr;
       while (true) {
-	*_pipe >> ptr;
-	if (!ptr) break;
-	*_pipe >> msgbuf1 >> msgbuf2 >> msgbuf3;
-	if (_verboseServer) cout << "RooRealMPFE::evaluate(" << GetName()
-	  << ") IPC fromServer> retrieving error log Arg " << ptr << " Msg " << msgbuf1 << endl ;
+   *_pipe >> ptr;
+   if (!ptr) break;
+   *_pipe >> msgbuf1 >> msgbuf2 >> msgbuf3;
+   if (_verboseServer) cout << "RooRealMPFE::evaluate(" << GetName()
+     << ") IPC fromServer> retrieving error log Arg " << ptr << " Msg " << msgbuf1 << endl ;
 
-	logEvalError(reinterpret_cast<RooAbsReal*>(ptr),msgbuf3,msgbuf1,msgbuf2) ;
+   logEvalError(reinterpret_cast<RooAbsReal*>(ptr),msgbuf3,msgbuf1,msgbuf2) ;
       }
       std::free(msgbuf1);
       std::free(msgbuf2);
@@ -601,7 +625,7 @@ Double_t RooRealMPFE::evaluate() const
     }
 
     // Mark end of calculation in progress
-    _calcInProgress = kFALSE ;
+    _calcInProgress = false ;
     return_value = value ;
 #endif // _WIN32
   }
@@ -623,26 +647,26 @@ void RooRealMPFE::standby()
     if (_pipe->good()) {
       // Terminate server process ;
       if (_verboseServer) cout << "RooRealMPFE::standby(" << GetName()
-	<< ") IPC toServer> Terminate " << endl;
+   << ") IPC toServer> Terminate " << endl;
       int msg = Terminate;
       *_pipe << msg << BidirMMapPipe::flush;
       // read handshake
       msg = 0;
       *_pipe >> msg;
       if (Terminate != msg || 0 != _pipe->close()) {
-	std::cerr << "In " << __func__ << "(" << __FILE__ ", " << __LINE__ <<
-	  "): Server shutdown failed." << std::endl;
+   std::cerr << "In " << __func__ << "(" << __FILE__ ", " << __LINE__ <<
+     "): Server shutdown failed." << std::endl;
       }
     } else {
       if (_verboseServer) {
-	std::cerr << "In " << __func__ << "(" << __FILE__ ", " <<
-	  __LINE__ << "): Pipe has already shut down, not sending "
-	  "Terminate to server." << std::endl;
+   std::cerr << "In " << __func__ << "(" << __FILE__ ", " <<
+     __LINE__ << "): Pipe has already shut down, not sending "
+     "Terminate to server." << std::endl;
       }
     }
     // Close pipes
     delete _pipe;
-    _pipe = 0;
+    _pipe = nullptr;
 
     // Revert to initialize state
     _state = Initialize;
@@ -656,7 +680,7 @@ void RooRealMPFE::standby()
 /// Intercept call to optimize constant term in test statistics
 /// and forward it to object on server side.
 
-void RooRealMPFE::constOptimizeTestStatistic(ConstOpCode opcode, Bool_t doAlsoTracking)
+void RooRealMPFE::constOptimizeTestStatistic(ConstOpCode opcode, bool doAlsoTracking)
 {
 #ifndef _WIN32
   if (_state==Client) {
@@ -665,7 +689,7 @@ void RooRealMPFE::constOptimizeTestStatistic(ConstOpCode opcode, Bool_t doAlsoTr
     int op = opcode;
     *_pipe << msg << op << doAlsoTracking;
     if (_verboseServer) cout << "RooRealMPFE::constOptimize(" << GetName()
-			     << ") IPC toServer> ConstOpt " << opcode << endl ;
+              << ") IPC toServer> ConstOpt " << opcode << endl ;
 
     initVars() ;
   }
@@ -682,14 +706,14 @@ void RooRealMPFE::constOptimizeTestStatistic(ConstOpCode opcode, Bool_t doAlsoTr
 /// Control verbose messaging related to inter process communication
 /// on both client and server side
 
-void RooRealMPFE::setVerbose(Bool_t clientFlag, Bool_t serverFlag)
+void RooRealMPFE::setVerbose(bool clientFlag, bool serverFlag)
 {
 #ifndef _WIN32
   if (_state==Client) {
     int msg = Verbose ;
     *_pipe << msg << serverFlag;
     if (_verboseServer) cout << "RooRealMPFE::setVerbose(" << GetName()
-			     << ") IPC toServer> Verbose " << (serverFlag?1:0) << endl ;
+              << ") IPC toServer> Verbose " << (serverFlag?1:0) << endl ;
   }
 #endif // _WIN32
   _verboseClient = clientFlag ; _verboseServer = serverFlag ;
@@ -700,14 +724,14 @@ void RooRealMPFE::setVerbose(Bool_t clientFlag, Bool_t serverFlag)
 /// Control verbose messaging related to inter process communication
 /// on both client and server side
 
-void RooRealMPFE::applyNLLWeightSquared(Bool_t flag)
+void RooRealMPFE::applyNLLWeightSquared(bool flag)
 {
 #ifndef _WIN32
   if (_state==Client) {
     int msg = ApplyNLLW2 ;
     *_pipe << msg << flag;
     if (_verboseServer) cout << "RooRealMPFE::applyNLLWeightSquared(" << GetName()
-			     << ") IPC toServer> ApplyNLLW2 " << (flag?1:0) << endl ;
+              << ") IPC toServer> ApplyNLLW2 " << (flag?1:0) << endl ;
   }
 #endif // _WIN32
   doApplyNLLW2(flag) ;
@@ -716,7 +740,7 @@ void RooRealMPFE::applyNLLWeightSquared(Bool_t flag)
 
 ////////////////////////////////////////////////////////////////////////////////
 
-void RooRealMPFE::doApplyNLLW2(Bool_t flag)
+void RooRealMPFE::doApplyNLLW2(bool flag)
 {
   RooNLLVar* nll = dynamic_cast<RooNLLVar*>(_arg.absArg()) ;
   if (nll) {
@@ -729,14 +753,14 @@ void RooRealMPFE::doApplyNLLW2(Bool_t flag)
 /// Control verbose messaging related to inter process communication
 /// on both client and server side
 
-void RooRealMPFE::enableOffsetting(Bool_t flag)
+void RooRealMPFE::enableOffsetting(bool flag)
 {
 #ifndef _WIN32
   if (_state==Client) {
     int msg = EnableOffset ;
     *_pipe << msg << flag;
     if (_verboseServer) cout << "RooRealMPFE::enableOffsetting(" << GetName()
-			     << ") IPC toServer> EnableOffset " << (flag?1:0) << endl ;
+              << ") IPC toServer> EnableOffset " << (flag?1:0) << endl ;
   }
 #endif // _WIN32
   ((RooAbsReal&)_arg.arg()).enableOffsetting(flag) ;
@@ -744,3 +768,34 @@ void RooRealMPFE::enableOffsetting(Bool_t flag)
 
 
 
+////////////////////////////////////////////////////////////////////////////////
+/// Destructor. Terminate all parallel processes still registered with
+/// the sentinel
+
+RooMPSentinel::~RooMPSentinel()
+{
+  for(auto * mpfe : static_range_cast<RooRealMPFE*>(_mpfeSet)) {
+    mpfe->standby() ;
+  }
+}
+
+
+
+////////////////////////////////////////////////////////////////////////////////
+/// Register given multi-processor front-end object with the sentinel
+
+void RooMPSentinel::add(RooRealMPFE& mpfe)
+{
+  _mpfeSet.add(mpfe,true) ;
+}
+
+
+
+////////////////////////////////////////////////////////////////////////////////
+/// Remove given multi-processor front-end object from the sentinel
+
+void RooMPSentinel::remove(RooRealMPFE& mpfe)
+{
+  _mpfeSet.remove(mpfe,true) ;
+}
+
