@@ -9,9 +9,10 @@ TEST(RNTuple, ReconstructModel)
    auto fieldKlass = model->MakeField<CustomStruct>("klass");
    auto fieldArray = model->MakeField<std::array<double, 2>>("array");
    auto fieldVariant = model->MakeField<std::variant<double, std::variant<std::string, double>>>("variant");
+   model->Freeze();
    {
       RPageSinkFile sink("myNTuple", fileGuard.GetPath(), RNTupleWriteOptions());
-      sink.Create(*model.get());
+      sink.Init(*model.get());
       sink.CommitClusterGroup();
       sink.CommitDataset();
       model = nullptr;
@@ -19,22 +20,30 @@ TEST(RNTuple, ReconstructModel)
 
    RPageSourceFile source("myNTuple", fileGuard.GetPath(), RNTupleReadOptions());
    source.Attach();
-
-   auto modelReconstructed = source.GetSharedDescriptorGuard()->GenerateModel();
    try {
-      modelReconstructed->GetDefaultEntry()->Get<float>("xyz");
+      source.SetEntryRange({0, source.GetNEntries() + 1});
+      FAIL() << "invalid entry range should throw";
+   } catch (const RException &err) {
+      EXPECT_THAT(err.what(), testing::HasSubstr("invalid entry range"));
+   }
+   // Should not throw
+   source.SetEntryRange({0, source.GetNEntries()});
+
+   auto modelReconstructed = source.GetSharedDescriptorGuard()->CreateModel();
+   try {
+      modelReconstructed->GetDefaultEntry().GetPtr<float>("xyz");
       FAIL() << "invalid field name should throw";
    } catch (const RException &err) {
       EXPECT_THAT(err.what(), testing::HasSubstr("invalid field name"));
    }
-   auto vecPtr = modelReconstructed->GetDefaultEntry()->Get<std::vector<std::vector<float>>>("nnlo");
+   auto vecPtr = modelReconstructed->GetDefaultEntry().GetPtr<std::vector<std::vector<float>>>("nnlo");
    EXPECT_TRUE(vecPtr != nullptr);
    // Don't crash
    vecPtr->push_back(std::vector<float>{1.0});
-   auto array = modelReconstructed->GetDefaultEntry()->Get<std::array<double, 2>>("array");
+   auto array = modelReconstructed->GetDefaultEntry().GetPtr<std::array<double, 2>>("array");
    EXPECT_TRUE(array != nullptr);
-   auto variant = modelReconstructed->GetDefaultEntry()->Get<
-      std::variant<double, std::variant<std::string, double>>>("variant");
+   auto variant =
+      modelReconstructed->GetDefaultEntry().GetPtr<std::variant<double, std::variant<std::string, double>>>("variant");
    EXPECT_TRUE(variant != nullptr);
 }
 
@@ -96,26 +105,25 @@ TEST(RNTuple, WriteRead)
    auto wrKlass = modelWrite->MakeField<CustomStruct>("klass");
    wrKlass->s = "abc";
 
+   modelWrite->Freeze();
    auto modelRead = modelWrite->Clone();
 
    {
-      RNTupleWriter ntuple(std::move(modelWrite),
-         std::make_unique<RPageSinkFile>("myNTuple", fileGuard.GetPath(), RNTupleWriteOptions()));
-      ntuple.Fill();
+      auto writer = RNTupleWriter::Recreate(std::move(modelWrite), "myNTuple", fileGuard.GetPath());
+      writer->Fill();
    }
 
-   auto rdSignal = modelRead->Get<bool>("signal");
-   auto rdPt = modelRead->Get<float>("pt");
-   auto rdEnergy = modelRead->Get<float>("energy");
-   auto rdTag = modelRead->Get<std::string>("tag");
-   auto rdJets = modelRead->Get<std::vector<float>>("jets");
-   auto rdNnlo = modelRead->Get<std::vector<std::vector<float>>>("nnlo");
-   auto rdKlass = modelRead->Get<CustomStruct>("klass");
+   auto rdSignal = modelRead->GetDefaultEntry().GetPtr<bool>("signal");
+   auto rdPt = modelRead->GetDefaultEntry().GetPtr<float>("pt");
+   auto rdEnergy = modelRead->GetDefaultEntry().GetPtr<float>("energy");
+   auto rdTag = modelRead->GetDefaultEntry().GetPtr<std::string>("tag");
+   auto rdJets = modelRead->GetDefaultEntry().GetPtr<std::vector<float>>("jets");
+   auto rdNnlo = modelRead->GetDefaultEntry().GetPtr<std::vector<std::vector<float>>>("nnlo");
+   auto rdKlass = modelRead->GetDefaultEntry().GetPtr<CustomStruct>("klass");
 
-   RNTupleReader ntuple(std::move(modelRead),
-      std::make_unique<RPageSourceFile>("myNTuple", fileGuard.GetPath(), RNTupleReadOptions()));
-   EXPECT_EQ(1U, ntuple.GetNEntries());
-   ntuple.LoadEntry(0);
+   auto reader = RNTupleReader::Open(std::move(modelRead), "myNTuple", fileGuard.GetPath());
+   EXPECT_EQ(1U, reader->GetNEntries());
+   reader->LoadEntry(0);
 
    EXPECT_TRUE(*rdSignal);
    EXPECT_EQ(42.0, *rdPt);
@@ -139,6 +147,48 @@ TEST(RNTuple, WriteRead)
    EXPECT_STREQ("abc", rdKlass->s.c_str());
 }
 
+TEST(RNTuple, WriteReadInlinedModel)
+{
+   FileRaii fileGuard("test_ntuple_writeread_inlinedmodel.root");
+
+   {
+      auto writer = RNTupleWriter::Recreate(
+         {
+            {"std::uint32_t", "id"},
+            {"std::vector<float>", "vpx"},
+            {"std::vector<float>", "vpy"},
+            {"std::vector<float>", "vpz"},
+         },
+         "NTuple", fileGuard.GetPath());
+
+      auto entry = writer->CreateEntry();
+      *entry->GetPtr<std::uint32_t>("id") = 1;
+      *entry->GetPtr<std::vector<float>>("vpx") = {1.0, 1.1, 1.2};
+      *entry->GetPtr<std::vector<float>>("vpy") = {2.0, 2.1, 2.2};
+      *entry->GetPtr<std::vector<float>>("vpz") = {3.0, 3.1, 3.2};
+
+      writer->Fill(*entry);
+   }
+
+   auto reader = RNTupleReader::Open("NTuple", fileGuard.GetPath());
+   EXPECT_EQ(1U, reader->GetNEntries());
+   reader->LoadEntry(0);
+   auto readid = reader->GetModel().GetDefaultEntry().GetPtr<std::uint32_t>("id");
+   EXPECT_EQ(1, *readid);
+   auto readvpx = reader->GetModel().GetDefaultEntry().GetPtr<std::vector<float>>("vpx");
+   EXPECT_FLOAT_EQ(1.0, (*readvpx)[0]);
+   EXPECT_FLOAT_EQ(1.1, (*readvpx)[1]);
+   EXPECT_FLOAT_EQ(1.2, (*readvpx)[2]);
+   auto readvpy = reader->GetModel().GetDefaultEntry().GetPtr<std::vector<float>>("vpy");
+   EXPECT_FLOAT_EQ(2.0, (*readvpy)[0]);
+   EXPECT_FLOAT_EQ(2.1, (*readvpy)[1]);
+   EXPECT_FLOAT_EQ(2.2, (*readvpy)[2]);
+   auto readvpz = reader->GetModel().GetDefaultEntry().GetPtr<std::vector<float>>("vpz");
+   EXPECT_FLOAT_EQ(3.0, (*readvpz)[0]);
+   EXPECT_FLOAT_EQ(3.1, (*readvpz)[1]);
+   EXPECT_FLOAT_EQ(3.2, (*readvpz)[2]);
+}
+
 TEST(RNTuple, FileAnchor)
 {
    FileRaii fileGuard("test_ntuple_file_anchor.root");
@@ -169,8 +219,8 @@ TEST(RNTuple, FileAnchor)
    EXPECT_EQ(1U, readerA->GetNEntries());
    EXPECT_EQ(1U, readerB->GetNEntries());
 
-   auto a = readerA->GetModel()->Get<int>("a");
-   auto b = readerB->GetModel()->Get<int>("b");
+   auto a = readerA->GetModel().GetDefaultEntry().GetPtr<int>("a");
+   auto b = readerB->GetModel().GetDefaultEntry().GetPtr<int>("b");
    readerA->LoadEntry(0);
    readerB->LoadEntry(0);
    EXPECT_EQ(42, *a);
@@ -194,35 +244,34 @@ TEST(RNTuple, Clusters)
    wrFourVec->at(2) = 2.0;
    wrFourVec->at(3) = 3.0;
 
+   modelWrite->Freeze();
    auto modelRead = modelWrite->Clone();
 
    {
-      RNTupleWriter ntuple(std::move(modelWrite),
-         std::make_unique<RPageSinkFile>("myNTuple", fileGuard.GetPath(), RNTupleWriteOptions()));
-      ntuple.Fill();
-      ntuple.CommitCluster();
+      auto writer = RNTupleWriter::Recreate(std::move(modelWrite), "myNTuple", fileGuard.GetPath());
+      writer->Fill();
+      writer->CommitCluster();
       *wrPt = 24.0;
       wrNnlo->clear();
       *wrTag = "";
       wrFourVec->at(2) = 42.0;
-      ntuple.Fill();
+      writer->Fill();
       *wrPt = 12.0;
       wrNnlo->push_back(std::vector<float>{42.0});
       *wrTag = "12345";
       wrFourVec->at(1) = 24.0;
-      ntuple.Fill();
+      writer->Fill();
    }
 
-   auto rdPt = modelRead->Get<float>("pt");
-   auto rdTag = modelRead->Get<std::string>("tag");
-   auto rdNnlo = modelRead->Get<std::vector<std::vector<float>>>("nnlo");
-   auto rdFourVec = modelRead->Get<std::array<float, 4>>("fourVec");
+   auto rdPt = modelRead->GetDefaultEntry().GetPtr<float>("pt");
+   auto rdTag = modelRead->GetDefaultEntry().GetPtr<std::string>("tag");
+   auto rdNnlo = modelRead->GetDefaultEntry().GetPtr<std::vector<std::vector<float>>>("nnlo");
+   auto rdFourVec = modelRead->GetDefaultEntry().GetPtr<std::array<float, 4>>("fourVec");
 
-   RNTupleReader ntuple(std::move(modelRead),
-      std::make_unique<RPageSourceFile>("myNTuple", fileGuard.GetPath(), RNTupleReadOptions()));
-   EXPECT_EQ(3U, ntuple.GetNEntries());
+   auto reader = RNTupleReader::Open(std::move(modelRead), "myNTuple", fileGuard.GetPath());
+   EXPECT_EQ(3U, reader->GetNEntries());
 
-   ntuple.LoadEntry(0);
+   reader->LoadEntry(0);
    EXPECT_EQ(42.0, *rdPt);
    EXPECT_STREQ("xyz", rdTag->c_str());
    EXPECT_EQ(3U, rdNnlo->size());
@@ -239,13 +288,13 @@ TEST(RNTuple, Clusters)
    EXPECT_EQ(2.0, (*rdFourVec)[2]);
    EXPECT_EQ(3.0, (*rdFourVec)[3]);
 
-   ntuple.LoadEntry(1);
+   reader->LoadEntry(1);
    EXPECT_EQ(24.0, *rdPt);
    EXPECT_STREQ("", rdTag->c_str());
    EXPECT_TRUE(rdNnlo->empty());
    EXPECT_EQ(42.0, (*rdFourVec)[2]);
 
-   ntuple.LoadEntry(2);
+   reader->LoadEntry(2);
    EXPECT_EQ(12.0, *rdPt);
    EXPECT_STREQ("12345", rdTag->c_str());
    EXPECT_EQ(1U, rdNnlo->size());
@@ -272,7 +321,7 @@ TEST(RNTuple, ClusterEntries)
 
    auto ntuple = RNTupleReader::Open("ntuple", fileGuard.GetPath());
    // 100 entries / 5 entries per cluster
-   EXPECT_EQ(20, ntuple->GetDescriptor()->GetNClusters());
+   EXPECT_EQ(20, ntuple->GetDescriptor().GetNClusters());
 }
 
 TEST(RNTuple, PageSize)
@@ -293,7 +342,7 @@ TEST(RNTuple, PageSize)
    }
 
    auto ntuple = RNTupleReader::Open("ntuple", fileGuard.GetPath());
-   const auto &col0_pages = ntuple->GetDescriptor()->GetClusterDescriptor(0).GetPageRange(0);
+   const auto &col0_pages = ntuple->GetDescriptor().GetClusterDescriptor(0).GetPageRange(0);
    // 1000 column elements / 50 elements per page
    EXPECT_EQ(20, col0_pages.fPageInfos.size());
 }
@@ -302,15 +351,7 @@ TEST(RNTupleModel, EnforceValidFieldNames)
 {
    auto model = RNTupleModel::Create();
 
-   auto field = model->MakeField<float>("pt", 42.0);
-
    // MakeField
-   try {
-      auto field2 = model->MakeField<float>("pt", 42.0);
-      FAIL() << "repeated field names should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("field name 'pt' already exists"));
-   }
    try {
       auto field3 = model->MakeField<float>("", 42.0);
       FAIL() << "empty string as field name should throw";
@@ -324,16 +365,19 @@ TEST(RNTupleModel, EnforceValidFieldNames)
       EXPECT_THAT(err.what(), testing::HasSubstr("name 'pt.pt' cannot contain dot characters '.'"));
    }
 
-   // AddField
+   // Previous failures to create 'pt' should not block the name
+   auto field = model->MakeField<float>("pt", 42.0);
+
    try {
-      model->AddField(std::make_unique<RField<float>>(RField<float>("pt")));
+      auto field2 = model->MakeField<float>("pt", 42.0);
       FAIL() << "repeated field names should throw";
-   } catch (const RException& err) {
+   } catch (const RException &err) {
       EXPECT_THAT(err.what(), testing::HasSubstr("field name 'pt' already exists"));
    }
+
+   // AddField
    try {
-      float num = 10.0;
-      model->AddField("pt", &num);
+      model->AddField(std::make_unique<RField<float>>("pt"));
       FAIL() << "repeated field names should throw";
    } catch (const RException& err) {
       EXPECT_THAT(err.what(), testing::HasSubstr("field name 'pt' already exists"));
@@ -361,9 +405,6 @@ TEST(RNTupleModel, FieldDescriptions)
 
    auto pt = model->MakeField<float>({"pt", "transverse momentum"}, 42.0);
 
-   float num = 10.0;
-   model->AddField({"mass", "mass"}, &num);
-
    auto charge = std::make_unique<RField<float>>(RField<float>("charge"));
    charge->SetDescription("electric charge");
    model->AddField(std::move(charge));
@@ -374,13 +415,12 @@ TEST(RNTupleModel, FieldDescriptions)
 
    auto ntuple = RNTupleReader::Open("ntuple", fileGuard.GetPath());
    std::vector<std::string> fieldDescriptions;
-   for (auto &f : ntuple->GetDescriptor()->GetTopLevelFields()) {
+   for (auto &f : ntuple->GetDescriptor().GetTopLevelFields()) {
       fieldDescriptions.push_back(f.GetFieldDescription());
    }
-   ASSERT_EQ(3, fieldDescriptions.size());
+   ASSERT_EQ(2u, fieldDescriptions.size());
    EXPECT_EQ(std::string("transverse momentum"), fieldDescriptions[0]);
-   EXPECT_EQ(std::string("mass"), fieldDescriptions[1]);
-   EXPECT_EQ(std::string("electric charge"), fieldDescriptions[2]);
+   EXPECT_EQ(std::string("electric charge"), fieldDescriptions[1]);
 }
 
 TEST(RNTupleModel, CollectionFieldDescriptions)
@@ -396,7 +436,7 @@ TEST(RNTupleModel, CollectionFieldDescriptions)
    }
 
    auto ntuple = RNTupleReader::Open("ntuple", fileGuard.GetPath());
-   const auto &muon_desc = *ntuple->GetDescriptor()->GetTopLevelFields().begin();
+   const auto &muon_desc = *ntuple->GetDescriptor().GetTopLevelFields().begin();
    EXPECT_EQ(std::string("muons after basic selection"), muon_desc.GetFieldDescription());
 }
 
@@ -405,16 +445,27 @@ TEST(RNTupleModel, GetField)
    auto m = RNTupleModel::Create();
    m->MakeField<int>("x");
    m->MakeField<CustomStruct>("cs");
-   EXPECT_EQ(m->GetField("x")->GetName(), "x");
-   EXPECT_EQ(m->GetField("x")->GetType(), "std::int32_t");
-   EXPECT_EQ(m->GetField("cs.v1")->GetName(), "v1");
-   EXPECT_EQ(m->GetField("cs.v1")->GetType(), "std::vector<float>");
-   EXPECT_EQ(m->GetField("nonexistent"), nullptr);
-   EXPECT_EQ(m->GetField(""), nullptr);
-   EXPECT_EQ("", m->GetFieldZero()->GetQualifiedFieldName());
-   EXPECT_EQ("x", m->GetField("x")->GetQualifiedFieldName());
-   EXPECT_EQ("cs", m->GetField("cs")->GetQualifiedFieldName());
-   EXPECT_EQ("cs.v1", m->GetField("cs.v1")->GetQualifiedFieldName());
+   m->Freeze();
+   EXPECT_EQ(m->GetField("x").GetFieldName(), "x");
+   EXPECT_EQ(m->GetField("x").GetTypeName(), "std::int32_t");
+   EXPECT_EQ(m->GetField("cs.v1").GetFieldName(), "v1");
+   EXPECT_EQ(m->GetField("cs.v1").GetTypeName(), "std::vector<float>");
+   try {
+      m->GetField("nonexistent");
+      FAIL() << "invalid field name should throw";
+   } catch (const RException &err) {
+      EXPECT_THAT(err.what(), testing::HasSubstr("invalid field"));
+   }
+   try {
+      m->GetField("");
+      FAIL() << "empty field name should throw";
+   } catch (const RException &err) {
+      EXPECT_THAT(err.what(), testing::HasSubstr("invalid field"));
+   }
+   EXPECT_EQ("", m->GetFieldZero().GetQualifiedFieldName());
+   EXPECT_EQ("x", m->GetField("x").GetQualifiedFieldName());
+   EXPECT_EQ("cs", m->GetField("cs").GetQualifiedFieldName());
+   EXPECT_EQ("cs.v1", m->GetField("cs.v1").GetQualifiedFieldName());
 }
 
 TEST(RNTuple, EmptyString)
@@ -466,67 +517,6 @@ TEST(RNTuple, NullSafety)
    } catch (const RException& err) {
       EXPECT_THAT(err.what(), testing::HasSubstr("null field"));
    }
-   try {
-      model->AddField<float>("pt", nullptr);
-      FAIL() << "null fields should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("null field fromWhere"));
-   }
-
-   // RNTupleReader and RNTupleWriter
-   FileRaii fileGuard("test_ntuple_null_safety.root");
-   try {
-      RNTupleWriter ntuple(nullptr,
-         std::make_unique<RPageSinkFile>("myNTuple", fileGuard.GetPath(), RNTupleWriteOptions()));
-      FAIL() << "null models should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("null model"));
-   }
-   try {
-      RNTupleWriter ntuple(std::move(model), nullptr);
-      FAIL() << "null sinks should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("null sink"));
-   }
-   try {
-      auto ntuple = RNTupleWriter::Recreate(nullptr, "myNtuple", fileGuard.GetPath());
-      FAIL() << "null models should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("null model"));
-   }
-   try {
-      auto file = std::make_unique<TFile>(fileGuard.GetPath().c_str(), "RECREATE", "", 101);
-      auto ntuple = RNTupleWriter::Append(nullptr, "myNtuple", *file);
-      FAIL() << "null models should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("null model"));
-   }
-
-   try {
-      auto ntuple = RNTupleReader::Open(nullptr, "myNTuple", fileGuard.GetPath());
-      FAIL() << "null models should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("null model"));
-   }
-   try {
-      RNTupleReader ntuple(nullptr,
-         std::make_unique<RPageSourceFile>("myNTuple", fileGuard.GetPath(), RNTupleReadOptions()));
-      FAIL() << "null models should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("null model"));
-   }
-   try {
-      RNTupleReader ntuple(RNTupleModel::Create(), nullptr);
-      FAIL() << "null sources should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("null source"));
-   }
-   try {
-      RNTupleReader ntuple(nullptr);
-      FAIL() << "null sources should throw";
-   } catch (const RException& err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("null source"));
-   }
 }
 
 TEST(RNTuple, ModelId)
@@ -534,7 +524,7 @@ TEST(RNTuple, ModelId)
    auto m1 = RNTupleModel::Create();
    auto m2 = RNTupleModel::Create();
    EXPECT_FALSE(m1->IsFrozen());
-   EXPECT_EQ(m1->GetModelId(), m2->GetModelId());
+   EXPECT_NE(m1->GetModelId(), m2->GetModelId());
 
    m1->Freeze();
    EXPECT_TRUE(m1->IsFrozen());
@@ -551,26 +541,15 @@ TEST(RNTuple, ModelId)
    } catch (const RException &err) {
       EXPECT_THAT(err.what(), testing::HasSubstr("invalid attempt to modify frozen model"));
    }
-   try {
-      float dummy;
-      m1->AddField<float>("pt", &dummy);
-      FAIL() << "changing frozen model should throw";
-   } catch (const RException &err) {
-      EXPECT_THAT(err.what(), testing::HasSubstr("invalid attempt to modify frozen model"));
-   }
 
-   EXPECT_NE(m1->GetModelId(), m2->GetModelId());
    // Freeze() should be idempotent call
    auto id = m1->GetModelId();
    m1->Freeze();
    EXPECT_TRUE(m1->IsFrozen());
    EXPECT_EQ(id, m1->GetModelId());
 
-   m2->Freeze();
-   EXPECT_NE(m1->GetModelId(), m2->GetModelId());
-
    auto m2c = m2->Clone();
-   EXPECT_EQ(m2->GetModelId(), m2c->GetModelId());
+   EXPECT_NE(m2->GetModelId(), m2c->GetModelId());
 }
 
 TEST(RNTuple, Entry)
@@ -610,27 +589,27 @@ TEST(RNTuple, BareEntry)
    FileRaii fileGuard("test_ntuple_bare_entry.root");
    {
       auto ntuple = RNTupleWriter::Recreate(std::move(m), "ntpl", fileGuard.GetPath());
-      const auto model = ntuple->GetModel();
+      const auto &model = ntuple->GetModel();
       try {
-         model->GetDefaultEntry();
+         model.GetDefaultEntry();
          FAIL() << "accessing default entry of bare model should throw";
       } catch (const RException &err) {
          EXPECT_THAT(err.what(), testing::HasSubstr("invalid attempt to use default entry of bare model"));
       }
       try {
-         model->Get<float>("pt");
+         model.GetDefaultEntry().GetPtr<float>("pt");
          FAIL() << "accessing default entry of bare model should throw";
       } catch (const RException &err) {
          EXPECT_THAT(err.what(), testing::HasSubstr("invalid attempt to use default entry of bare model"));
       }
 
-      auto e1 = model->CreateEntry();
-      ASSERT_NE(nullptr, e1->Get<float>("pt"));
-      *(e1->Get<float>("pt")) = 1.0;
-      auto e2 = model->CreateBareEntry();
-      EXPECT_EQ(nullptr, e2->Get<float>("pt"));
+      auto e1 = model.CreateEntry();
+      ASSERT_NE(nullptr, e1->GetPtr<float>("pt"));
+      *(e1->GetPtr<float>("pt")) = 1.0;
+      auto e2 = model.CreateBareEntry();
+      EXPECT_EQ(nullptr, e2->GetPtr<float>("pt"));
       float pt = 2.0;
-      e2->CaptureValueUnsafe("pt", &pt);
+      e2->BindRawPtr("pt", &pt);
 
       ntuple->Fill(*e1);
       ntuple->Fill(*e2);
@@ -639,22 +618,22 @@ TEST(RNTuple, BareEntry)
    auto ntuple = RNTupleReader::Open("ntpl", fileGuard.GetPath());
    ASSERT_EQ(2U, ntuple->GetNEntries());
    ntuple->LoadEntry(0);
-   EXPECT_EQ(1.0, *ntuple->GetModel()->GetDefaultEntry()->Get<float>("pt"));
+   EXPECT_EQ(1.0, *ntuple->GetModel().GetDefaultEntry().GetPtr<float>("pt"));
    ntuple->LoadEntry(1);
-   EXPECT_EQ(2.0, *ntuple->GetModel()->GetDefaultEntry()->Get<float>("pt"));
+   EXPECT_EQ(2.0, *ntuple->GetModel().GetDefaultEntry().GetPtr<float>("pt"));
 }
 
 namespace ROOT::Experimental::Internal {
 struct RFieldCallbackInjector {
    template <typename FieldT>
-   static void Inject(FieldT &field, ROOT::Experimental::Detail::RFieldBase::ReadCallback_t func)
+   static void Inject(FieldT &field, ROOT::Experimental::RFieldBase::ReadCallback_t func)
    {
       field.AddReadCallback(func);
    }
 };
 } // namespace ROOT::Experimental::Internal
 namespace {
-static unsigned gNCallReadCallback = 0;
+unsigned gNCallReadCallback = 0;
 }
 
 TEST(RNTuple, ReadCallback)
@@ -693,7 +672,7 @@ TEST(RNTuple, ReadCallback)
    model->AddField(std::move(fieldKlass));
 
    auto ntuple = RNTupleReader::Open(std::move(model), "f", fileGuard.GetPath());
-   auto rdKlass = ntuple->GetModel()->GetDefaultEntry()->Get<CustomStruct>("klass");
+   auto rdKlass = ntuple->GetModel().GetDefaultEntry().GetPtr<CustomStruct>("klass");
    EXPECT_EQ(2U, ntuple->GetNEntries());
    ntuple->LoadEntry(0);
    EXPECT_EQ(1337.0, rdKlass->a);
@@ -732,3 +711,120 @@ TEST(RNTuple, FillBytesWritten)
    optsSmall.SetHasSmallClusters(true);
    checkFillReturnValue(optsSmall);
 }
+
+TEST(RNTuple, FillBytesWrittenCollections)
+{
+   FileRaii fileGuard("test_ntuple_fillbytes_collections.root");
+
+   auto eventModel = RNTupleModel::Create();
+   auto fldPt = eventModel->MakeField<float>("pt", 0.0);
+
+   auto hitModel = RNTupleModel::Create();
+   auto fldHitX = hitModel->MakeField<float>("x", 0.0);
+   auto fldHitY = hitModel->MakeField<float>("y", 0.0);
+
+   auto trackModel = RNTupleModel::Create();
+   auto fldTrackEnergy = trackModel->MakeField<float>("energy", 0.0);
+
+   auto fldHits = trackModel->MakeCollection("hits", std::move(hitModel));
+   auto fldTracks = eventModel->MakeCollection("tracks", std::move(trackModel));
+
+   {
+      RNTupleWriteOptions options;
+      options.SetApproxZippedClusterSize(1024 * 1024);
+      auto writer = RNTupleWriter::Recreate(std::move(eventModel), "myNTuple", fileGuard.GetPath(), options);
+
+      for (unsigned i = 0; i < 30000; ++i) {
+         for (unsigned t = 0; t < 3; ++t) {
+            for (unsigned h = 0; h < 2; ++h) {
+               *fldHitX = 4.0;
+               *fldHitY = 8.0;
+               EXPECT_EQ(4 + 4, fldHits->Fill());
+            }
+            *fldTrackEnergy = i * t;
+            EXPECT_EQ(2 * (4 + 4) + 8 + 4, fldTracks->Fill());
+         }
+         *fldPt = float(i);
+         EXPECT_EQ(3 * (2 * (4 + 4) + 8 + 4) + 8 + 4, writer->Fill());
+      }
+   }
+
+   auto reader = RNTupleReader::Open("myNTuple", fileGuard.GetPath());
+   EXPECT_EQ(reader->GetDescriptor().GetNClusters(), 2);
+}
+
+TEST(RNTuple, RValue)
+{
+   auto f1 = RFieldBase::Create("f1", "CustomStruct").Unwrap();
+   auto f2 = RFieldBase::Create("f2", "std::vector<std::vector<std::variant<std::string, float>>>").Unwrap();
+   auto f3 = RFieldBase::Create("f3", "std::variant<std::unique_ptr<CustomStruct>, ComplexStruct>>").Unwrap();
+   auto v1 = f1->CreateValue();
+   auto v2 = f2->CreateValue();
+   auto v3 = f3->CreateValue();
+
+   auto p = v3.GetPtr<void>();
+   v3.EmplaceNew();
+   EXPECT_NE(p.get(), v3.GetPtr<void>().get());
+
+   // Destruct fields to check if the deleters work without the fields
+   f1 = nullptr;
+   f2 = nullptr;
+   f3 = nullptr;
+
+   // The deleters are called in the destructors of the values
+}
+
+TEST(REntry, Basics)
+{
+   auto model = RNTupleModel::Create();
+   model->MakeField<float>("pt");
+   model->Freeze();
+
+   auto e = model->CreateEntry();
+   EXPECT_EQ(e->GetModelId(), model->GetModelId());
+   for (const auto &v : *e) {
+      EXPECT_STREQ("pt", v.GetField().GetFieldName().c_str());
+   }
+
+   EXPECT_THROW(e->GetToken(""), ROOT::Experimental::RException);
+   EXPECT_THROW(e->GetToken("eta"), ROOT::Experimental::RException);
+   EXPECT_THROW(model->GetToken("eta"), ROOT::Experimental::RException);
+
+   std::shared_ptr<float> ptrPt;
+   e->BindValue("pt", ptrPt);
+   EXPECT_EQ(ptrPt.get(), e->GetPtr<float>("pt").get());
+
+   auto model2 = model->Clone();
+   EXPECT_THROW(e->GetPtr<void>(model2->GetDefaultEntry().GetToken("pt")), ROOT::Experimental::RException);
+   std::shared_ptr<double> ptrDouble;
+   EXPECT_THROW(e->BindValue("pt", ptrDouble), ROOT::Experimental::RException);
+
+   float pt;
+   e->BindRawPtr("pt", &pt);
+   EXPECT_EQ(&pt, e->GetPtr<void>("pt").get());
+
+   e->EmplaceNewValue(model->GetToken("pt"));
+   EXPECT_NE(&pt, e->GetPtr<void>("pt").get());
+}
+
+TEST(RFieldBase, CreateObject)
+{
+   auto ptrInt = RField<int>("name").CreateObject<int>();
+   EXPECT_EQ(0, *ptrInt);
+
+   void *rawPtr = RField<int>("name").CreateObject<void>().release();
+   EXPECT_EQ(0, *static_cast<int *>(rawPtr));
+   delete static_cast<int *>(rawPtr);
+
+   {
+      ROOT::TestSupport::CheckDiagsRAII diagRAII;
+      diagRAII.requiredDiag(kWarning, "[ROOT.NTuple]", "possibly leaking", false /* matchFullMessage */);
+      auto p = RField<int>("name").CreateObject<void>();
+      delete static_cast<int *>(p.get()); // avoid release
+   }
+
+   EXPECT_THROW(RField<int>("name").CreateObject<float>(), RException);
+
+   auto ptrClass = RField<LowPrecisionFloats>("name").CreateObject<LowPrecisionFloats>();
+   EXPECT_DOUBLE_EQ(1.0, ptrClass->b);
+}
